"""
Test that we page getting a long backtrace on more than one thread
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestThreadBacktracePage(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_thread_backtrace_one_thread(self):
        """Run a simplified version of the test that just hits one breakpoint and
        doesn't care about synchronizing the two threads - hopefully this will
        run on more systems."""
        self.build()
        (
            self.inferior_target,
            self.process,
            thread,
            bkpt,
        ) = lldbutil.run_to_source_breakpoint(
            self, self.bkpt_string, lldb.SBFileSpec("main.cpp"), only_one_thread=False
        )

        # We hit the breakpoint on at least one thread.  If we hit it on both threads
        # simultaneously, we are ready to run our tests.  Otherwise, suspend the thread
        # that hit the breakpoint, and continue till the second thread hits
        # the breakpoint:

        (breakpoint_threads, other_threads) = ([], [])
        lldbutil.sort_stopped_threads(
            self.process,
            breakpoint_threads=breakpoint_threads,
            other_threads=other_threads,
        )
        self.assertGreater(
            len(breakpoint_threads), 0, "We hit at least one breakpoint thread"
        )
        self.assertGreater(len(breakpoint_threads[0].frames), 2, "I can go up")
        thread_id = breakpoint_threads[0].idx
        name = breakpoint_threads[0].frame[1].name.split("(")[0]
        self.check_one_thread(thread_id, name)

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number for our breakpoint.
        self.bkpt_string = "// Set breakpoint here"

    def check_one_thread(self, thread_id, func_name):
        # Now issue some thread backtrace commands and make sure they
        # get the right answer back.
        interp = self.dbg.GetCommandInterpreter()
        result = lldb.SBCommandReturnObject()

        # Run the real backtrace, remember to pass True for add_to_history since
        # we don't generate repeat commands for commands that aren't going into the history.
        interp.HandleCommand(
            "thread backtrace --count 10 {0}".format(thread_id), result, True
        )
        self.assertTrue(result.Succeeded(), "bt with count succeeded")
        # There should be 11 lines:
        lines = result.GetOutput().splitlines()
        self.assertEqual(len(lines), 11, "Got the right number of lines")
        # First frame is stop_here:
        self.assertNotEqual(lines[1].find("stop_here"), -1, "Found Stop Here")
        for line in lines[2:10]:
            self.assertNotEqual(
                line.find(func_name),
                -1,
                "Name {0} not found in line: {1}".format(func_name, line),
            )
        # The last entry should be 43:
        self.assertNotEqual(lines[10].find("count=43"), -1, "First show ends at 43")

        # Now try a repeat, and make sure we get 10 more on this thread:
        # import pdb; pdb.set_trace()
        interp.HandleCommand("", result, True)
        self.assertTrue(
            result.Succeeded(), "repeat command failed: {0}".format(result.GetError())
        )
        lines = result.GetOutput().splitlines()
        self.assertEqual(len(lines), 11, "Repeat got 11 lines")
        # Every line should now be the recurse function:
        for line in lines[1:10]:
            self.assertNotEqual(line.find(func_name), -1, "Name in every line")
        self.assertNotEqual(lines[10].find("count=33"), -1, "Last one is now 33")

    def check_two_threads(
        self, result_str, thread_id_1, name_1, thread_id_2, name_2, start_idx, end_idx
    ):
        # We should have 2 occurrences ot the thread header:
        self.assertEqual(
            result_str.count("thread #{0}".format(thread_id_1)), 1, "One for thread 1"
        )
        self.assertEqual(
            result_str.count("thread #{0}".format(thread_id_2)), 1, "One for thread 2"
        )
        # We should have 10 occurrences of each name:
        self.assertEqual(result_str.count(name_1), 10, "Found 10 of {0}".format(name_1))
        self.assertEqual(result_str.count(name_2), 10, "Found 10 of {0}".format(name_1))
        # There should be two instances of count=<start_idx> and none of count=<start-1>:
        self.assertEqual(
            result_str.count("count={0}".format(start_idx)),
            2,
            "Two instances of start_idx",
        )
        self.assertEqual(
            result_str.count("count={0}".format(start_idx - 1)),
            0,
            "No instances of start_idx - 1",
        )
        # There should be two instances of count=<end_idx> and none of count=<end_idx+1>:
        self.assertEqual(
            result_str.count("count={0}".format(end_idx)), 2, "Two instances of end_idx"
        )
        self.assertEqual(
            result_str.count("count={0}".format(end_idx + 1)),
            0,
            "No instances after end idx",
        )

    # The setup of this test was copied from the step-out test, and I can't tell from
    # the comments whether it was getting two threads to the same breakpoint that was
    # problematic, or the step-out part.  This test stops at the rendevous point so I'm
    # removing the skipIfLinux to see if we see any flakiness in just this part of the test.
    @skipIfWindows  # This test will hang on windows llvm.org/pr21753
    @expectedFailureAll(oslist=["windows"])
    @expectedFailureNetBSD
    def test_thread_backtrace_two_threads(self):
        """Test that repeat works even when backtracing on more than one thread."""
        self.build()
        (
            self.inferior_target,
            self.process,
            thread,
            bkpt,
        ) = lldbutil.run_to_source_breakpoint(
            self, self.bkpt_string, lldb.SBFileSpec("main.cpp"), only_one_thread=False
        )

        # We hit the breakpoint on at least one thread.  If we hit it on both threads
        # simultaneously, we are ready to run our tests.  Otherwise, suspend the thread
        # that hit the breakpoint, and continue till the second thread hits
        # the breakpoint:

        (breakpoint_threads, other_threads) = ([], [])
        lldbutil.sort_stopped_threads(
            self.process,
            breakpoint_threads=breakpoint_threads,
            other_threads=other_threads,
        )
        if len(breakpoint_threads) == 1:
            success = thread.Suspend()
            self.assertTrue(success, "Couldn't suspend a thread")
            breakpoint_threads = lldbutil.continue_to_breakpoint(self.process, bkpt)
            self.assertEqual(len(breakpoint_threads), 2, "Second thread stopped")

        # Figure out which thread is which:
        thread_id_1 = breakpoint_threads[0].idx
        self.assertGreater(len(breakpoint_threads[0].frames), 2, "I can go up")
        name_1 = breakpoint_threads[0].frame[1].name.split("(")[0]

        thread_id_2 = breakpoint_threads[1].idx
        self.assertGreater(len(breakpoint_threads[1].frames), 2, "I can go up")
        name_2 = breakpoint_threads[1].frame[1].name.split("(")[0]

        # Check that backtrace and repeat works on one thread, then works on the second
        # when we switch to it:
        self.check_one_thread(thread_id_1, name_1)
        self.check_one_thread(thread_id_2, name_2)

        # The output is looking right at this point, let's just do a couple more quick checks
        # to see we handle two threads and a start count:
        interp = self.dbg.GetCommandInterpreter()
        result = lldb.SBCommandReturnObject()

        interp.HandleCommand(
            "thread backtrace --count 10 --start 10 {0} {1}".format(
                thread_id_1, thread_id_2
            ),
            result,
            True,
        )
        self.assertTrue(result.Succeeded(), "command succeeded for two threads")

        result.Clear()
        interp.HandleCommand("", result, True)
        self.assertTrue(result.Succeeded(), "repeat command succeeded for two threads")
        result_str = result.GetOutput()
        self.check_two_threads(
            result_str, thread_id_1, name_1, thread_id_2, name_2, 23, 32
        )

        # Finally make sure the repeat repeats:
        result.Clear()
        interp.HandleCommand("", result, True)
        self.assertTrue(result.Succeeded(), "repeat command succeeded for two threads")
        result_str = result.GetOutput()
        self.check_two_threads(
            result_str, thread_id_1, name_1, thread_id_2, name_2, 13, 22
        )



# RUN: %PYTHON %s 2>&1 | FileCheck %s

import ctypes
import sys
from mlir.ir import *
from mlir.dialects import builtin
from mlir.dialects import func
from mlir.dialects import linalg
from mlir.passmanager import *
from mlir.execution_engine import *

from mlir.dialects.linalg.opdsl.lang import *


# Log everything to stderr and flush so that we have a unified stream to match
# errors/info emitted by MLIR to stderr.
def log(*args):
    print(*args, file=sys.stderr)
    sys.stderr.flush()


fill_boiler = """
func.func @main() -> i32 attributes {llvm.emit_c_interface} {
  %O0 = memref.alloc() : memref<i32>
  %O1 = memref.alloc() : memref<16xi32>
  %O2 = memref.alloc() : memref<4x16xi32>

  %val0 = arith.constant 1 : i32
  %val1 = arith.constant 2 : i32
  %val2 = arith.constant 3 : i32

  call @fill_0d_on_buffers(%val0, %O0) : (i32, memref<i32>) -> ()
  call @fill_1d_on_buffers(%val1, %O1) : (i32, memref<16xi32>) -> ()
  call @fill_2d_on_buffers(%val2, %O2) : (i32, memref<4x16xi32>) -> ()

  %c0 = arith.constant 0 : index
  %res0 = memref.load %O0[] : memref<i32>
  %c8 = arith.constant 8 : index
  %res1 = memref.load %O1[%c8] : memref<16xi32>
  %c2 = arith.constant 2 : index
  %res2 = memref.load %O2[%c2, %c8] : memref<4x16xi32>

  %0 = arith.addi %res0, %res1 : i32
  %1 = arith.addi %0, %res2 : i32

  // TODO: FFI-based solution to allow testing and printing with python code.
  return %1 : i32
}
"""

fill_rng_boiler = """
func.func @main() -> i32 attributes {llvm.emit_c_interface} {
  %O = memref.alloc() : memref<4x16xi32>
  %min = arith.constant -1000.0 : f64
  %max = arith.constant 1000.0 : f64
  %seed = arith.constant 42 : i32

  call @fill_rng_on_buffers(%min, %max, %seed, %O) :
    (f64, f64, i32, memref<4x16xi32>) -> ()

  %c0 = arith.constant 0 : index
  %0 = memref.load %O[%c0, %c0] : memref<4x16xi32>

  // TODO: FFI-based solution to allow testing and printing with python code.
  return %0 : i32
}
"""

conv_boiler = """
func.func @main() -> i32 attributes {llvm.emit_c_interface} {
  %v0 = arith.constant 0 : i32
  %v1 = arith.constant 1.0 : f64
  %v2 = arith.constant 2.0 : f64

  %input = memref.alloc() : memref<1x4x16x1xf64>
  %filter = memref.alloc() : memref<2x2x1xf64>
  %output = memref.alloc() : memref<1x2x4x1xi32>
  linalg.fill ins(%v1 : f64) outs(%input : memref<1x4x16x1xf64>)
  linalg.fill ins(%v2 : f64) outs(%filter : memref<2x2x1xf64>)
  linalg.fill ins(%v0 : i32) outs(%output : memref<1x2x4x1xi32>)

  call @conv_on_buffers(%input, %filter, %output) :
    (memref<1x4x16x1xf64>, memref<2x2x1xf64>, memref<1x2x4x1xi32>) -> ()

  %c0 = arith.constant 0 : index
  %0 = memref.load %output[%c0, %c0, %c0, %c0] : memref<1x2x4x1xi32>

  // TODO: FFI-based solution to allow testing and printing with python code.
  return %0 : i32
}
"""

pooling_boiler = """
func.func @main() -> i32 attributes {llvm.emit_c_interface} {
  %v0 = arith.constant 0 : i32
  %v42 = arith.constant 42.0 : f64
  %v77 = arith.constant 77.0 : f64
  %v-13 = arith.constant -13.0 : f64
  %v1 = arith.constant 1.0 : f64

  %input = memref.alloc() : memref<1x4x16x1xf64>
  %shape = memref.alloc() : memref<2x2xf64>
  %output = memref.alloc() : memref<1x2x4x1xi32>
  linalg.fill ins(%v1 : f64) outs(%input : memref<1x4x16x1xf64>)
  linalg.fill ins(%v1 : f64) outs(%shape : memref<2x2xf64>)
  linalg.fill ins(%v0 : i32) outs(%output : memref<1x2x4x1xi32>)

  %c0 = arith.constant 0 : index
  %c1 = arith.constant 1 : index
  %c2 = arith.constant 2 : index
  memref.store %v42, %input[%c0, %c0, %c0, %c0] : memref<1x4x16x1xf64>
  memref.store %v77, %input[%c0, %c0, %c1, %c0] : memref<1x4x16x1xf64>
  memref.store %v-13, %input[%c0, %c1, %c0, %c0] : memref<1x4x16x1xf64>

  call @pooling_on_buffers(%input, %shape, %output) :
    (memref<1x4x16x1xf64>, memref<2x2xf64>, memref<1x2x4x1xi32>) -> ()

  %0 = memref.load %output[%c0, %c0, %c0, %c0] : memref<1x2x4x1xi32>

  // TODO: FFI-based solution to allow testing and printing with python code.
  return %0 : i32
}
"""


def transform(module, boilerplate):
    # TODO: Allow cloning functions from one module to another.
    # Atm we have to resort to string concatenation.
    ops = module.operation.regions[0].blocks[0].operations
    mod = Module.parse("\n".join([str(op) for op in ops]) + boilerplate)

    pm = PassManager("builtin.module")
    pm.add("func.func(convert-linalg-to-loops)")
    pm.add("func.func(lower-affine)")
    pm.add("func.func(convert-math-to-llvm)")
    pm.add("func.func(convert-scf-to-cf)")
    pm.add("func.func(arith-expand)")
    pm.add("func.func(memref-expand)")
    pm.add("convert-vector-to-llvm")
    pm.add("finalize-memref-to-llvm")
    pm.add("convert-func-to-llvm")
    pm.add("convert-arith-to-llvm")
    pm.add("convert-cf-to-llvm")
    pm.add("reconcile-unrealized-casts")
    pm.run(mod.operation)
    return mod


def test_fill_builtin():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        i32 = IntegerType.get_signless(32)
        with InsertionPoint(module.body):

            @func.FuncOp.from_py_func(i32, MemRefType.get([], i32))
            def fill_0d_on_buffers(value, out):
                linalg.fill(value, outs=[out])

            @func.FuncOp.from_py_func(i32, MemRefType.get([16], i32))
            def fill_1d_on_buffers(value, out):
                linalg.fill(value, outs=[out])

            @func.FuncOp.from_py_func(i32, MemRefType.get([4, 16], i32))
            def fill_2d_on_buffers(value, out):
                linalg.fill(value, outs=[out])

        execution_engine = ExecutionEngine(transform(module, fill_boiler))

        # TODO: FFI-based solution to allow testing and printing with python code.
        # Prepare arguments: one result i32.
        # Arguments must be passed as pointers.
        c_int_p = ctypes.c_int * 1
        res = c_int_p(-1)
        execution_engine.invoke("main", res)

        log("RESULT: ", res[0])
        # CHECK: RESULT: 6


test_fill_builtin()


def test_fill_generic():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        i32 = IntegerType.get_signless(32)
        with InsertionPoint(module.body):

            @func.FuncOp.from_py_func(i32, MemRefType.get([], i32))
            def fill_0d_on_buffers(value, out):
                linalg.fill(value, outs=[out], emit_generic=True)

            @func.FuncOp.from_py_func(i32, MemRefType.get([16], i32))
            def fill_1d_on_buffers(value, out):
                linalg.fill(value, outs=[out], emit_generic=True)

            @func.FuncOp.from_py_func(i32, MemRefType.get([4, 16], i32))
            def fill_2d_on_buffers(value, out):
                linalg.fill(value, outs=[out], emit_generic=True)

        execution_engine = ExecutionEngine(transform(module, fill_boiler))

        # TODO: FFI-based solution to allow testing and printing with python code.
        # Prepare arguments: one result i32.
        # Arguments must be passed as pointers.
        c_int_p = ctypes.c_int * 1
        res = c_int_p(-1)
        execution_engine.invoke("main", res)

        log("RESULT: ", res[0])
        # CHECK: RESULT: 6


test_fill_generic()


def test_fill_rng_builtin():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        f64 = F64Type.get()
        i32 = IntegerType.get_signless(32)
        with InsertionPoint(module.body):

            @func.FuncOp.from_py_func(f64, f64, i32, MemRefType.get((4, 16), i32))
            def fill_rng_on_buffers(min, max, seed, out):
                linalg.fill_rng_2d(min, max, seed, outs=[out])

        execution_engine = ExecutionEngine(transform(module, fill_rng_boiler))

        # TODO: FFI-based solution to allow testing and printing with python code.
        # Prepare arguments: one result i32.
        # Arguments must be passed as pointers.
        c_int_p = ctypes.c_int * 1
        res = c_int_p(-1)
        execution_engine.invoke("main", res)

        log("RESULT: ", res[0])
        # CHECK: RESULT: -480


test_fill_rng_builtin()


def test_fill_rng_generic():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        f64 = F64Type.get()
        i32 = IntegerType.get_signless(32)
        with InsertionPoint(module.body):

            @func.FuncOp.from_py_func(f64, f64, i32, MemRefType.get((4, 16), i32))
            def fill_rng_on_buffers(min, max, seed, out):
                linalg.fill_rng_2d(min, max, seed, outs=[out], emit_generic=True)

        execution_engine = ExecutionEngine(transform(module, fill_rng_boiler))

        # TODO: FFI-based solution to allow testing and printing with python code.
        # Prepare arguments: one result i32.
        # Arguments must be passed as pointers.
        c_int_p = ctypes.c_int * 1
        res = c_int_p(-1)
        execution_engine.invoke("main", res)

        log("RESULT: ", res[0])
        # CHECK: RESULT: -480


test_fill_rng_generic()


def test_max_pooling_builtin():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        f64 = F64Type.get()
        i32 = IntegerType.get_signless(32)
        with InsertionPoint(module.body):

            @func.FuncOp.from_py_func(
                MemRefType.get((1, 4, 16, 1), f64),
                MemRefType.get((2, 2), f64),
                MemRefType.get((1, 2, 4, 1), i32),
            )
            def pooling_on_buffers(input, shape, output):
                linalg.pooling_nhwc_max(
                    input, shape, outs=[output], strides=[2, 4], dilations=[1, 2]
                )

        execution_engine = ExecutionEngine(transform(module, pooling_boiler))

        # TODO: FFI-based solution to allow testing and printing with python code.
        # Prepare arguments: one result i32.
        # Arguments must be passed as pointers.
        c_int_p = ctypes.c_int * 1
        res = c_int_p(-1)
        execution_engine.invoke("main", res)

        log("RESULT: ", res[0])
        # 77 is not selected due to the dilation 2 in the second dimension.
        # CHECK: RESULT: 42


test_max_pooling_builtin()


def test_max_pooling_generic():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        f64 = F64Type.get()
        i32 = IntegerType.get_signless(32)
        with InsertionPoint(module.body):

            @func.FuncOp.from_py_func(
                MemRefType.get((1, 4, 16, 1), f64),
                MemRefType.get((2, 2), f64),
                MemRefType.get((1, 2, 4, 1), i32),
            )
            def pooling_on_buffers(input, shape, output):
                linalg.pooling_nhwc_max(
                    input,
                    shape,
                    outs=[output],
                    strides=[2, 4],
                    dilations=[1, 2],
                    emit_generic=True,
                )

        execution_engine = ExecutionEngine(transform(module, pooling_boiler))

        # TODO: FFI-based solution to allow testing and printing with python code.
        # Prepare arguments: one result i32.
        # Arguments must be passed as pointers.
        c_int_p = ctypes.c_int * 1
        res = c_int_p(-1)
        execution_engine.invoke("main", res)

        log("RESULT: ", res[0])
        # 77 is not selected due to the dilation 2 in the second dimension.
        # CHECK: RESULT: 42


test_max_pooling_generic()


def test_min_pooling_builtin():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        f64 = F64Type.get()
        i32 = IntegerType.get_signless(32)
        with InsertionPoint(module.body):

            @func.FuncOp.from_py_func(
                MemRefType.get((1, 4, 16, 1), f64),
                MemRefType.get((2, 2), f64),
                MemRefType.get((1, 2, 4, 1), i32),
            )
            # Set the strides and use the default dilations.
            def pooling_on_buffers(input, shape, output):
                linalg.pooling_nhwc_min(input, shape, outs=[output], strides=[2, 4])

        execution_engine = ExecutionEngine(transform(module, pooling_boiler))

        # TODO: FFI-based solution to allow testing and printing with python code.
        # Prepare arguments: one result i32.
        # Arguments must be passed as pointers.
        c_int_p = ctypes.c_int * 1
        res = c_int_p(-1)
        execution_engine.invoke("main", res)

        log("RESULT: ", res[0])
        # CHECK: RESULT: -13


test_min_pooling_builtin()


def test_min_pooling_generic():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        f64 = F64Type.get()
        i32 = IntegerType.get_signless(32)
        with InsertionPoint(module.body):

            @func.FuncOp.from_py_func(
                MemRefType.get((1, 4, 16, 1), f64),
                MemRefType.get((2, 2), f64),
                MemRefType.get((1, 2, 4, 1), i32),
            )
            # Set the strides and use the default dilations.
            def pooling_on_buffers(input, shape, output):
                linalg.pooling_nhwc_min(
                    input, shape, outs=[output], strides=[2, 4], emit_generic=True
                )

        execution_engine = ExecutionEngine(transform(module, pooling_boiler))

        # TODO: FFI-based solution to allow testing and printing with python code.
        # Prepare arguments: one result i32.
        # Arguments must be passed as pointers.
        c_int_p = ctypes.c_int * 1
        res = c_int_p(-1)
        execution_engine.invoke("main", res)

        log("RESULT: ", res[0])
        # CHECK: RESULT: -13


test_min_pooling_generic()



from lldbsuite.test.decorators import *
from lldbsuite.test.concurrent_base import ConcurrentEventsBase
from lldbsuite.test.lldbtest import TestBase


@skipIfWindows
class ConcurrentBreakpointDelayBreakpointOneSignal(ConcurrentEventsBase):
    # Atomic sequences are not supported yet for MIPS in LLDB.
    @skipIf(triple="^mips")
    def test(self):
        """Test two threads that trigger a breakpoint (one with a 1 second delay) and one signal thread."""
        self.build()
        self.do_thread_actions(
            num_breakpoint_threads=1,
            num_delay_breakpoint_threads=1,
            num_signal_threads=1,
        )



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class TestContinue(GDBRemoteTestBase):
    class BaseResponder(MockGDBServerResponder):
        def qSupported(self, client_supported):
            return "PacketSize=3fff;QStartNoAckMode+;multiprocess+"

        def qfThreadInfo(self):
            return "mp400.401"

        def haltReason(self):
            return "S13"

        def cont(self):
            return "W01"

        def other(self, packet):
            if packet == "vCont?":
                return "vCont;c;C;s;S"
            if packet.startswith("vCont;"):
                return "W00"
            return ""

    def test_continue_no_multiprocess(self):
        class MyResponder(self.BaseResponder):
            def qSupported(self, client_supported):
                return "PacketSize=3fff;QStartNoAckMode+"

            def qfThreadInfo(self):
                return "m401"

        self.server.responder = MyResponder()
        self.runCmd("platform select remote-linux")
        target = self.createTarget("a.yaml")
        process = self.connect(target)
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateExited]
        )
        self.assertPacketLogReceived(["vCont;C13:401"])

    def test_continue_no_vCont(self):
        class MyResponder(self.BaseResponder):
            def qSupported(self, client_supported):
                return "PacketSize=3fff;QStartNoAckMode+"

            def qfThreadInfo(self):
                return "m401"

            def other(self, packet):
                return ""

        self.server.responder = MyResponder()
        self.runCmd("platform select remote-linux")
        target = self.createTarget("a.yaml")
        process = self.connect(target)
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateExited]
        )
        self.assertPacketLogReceived(["Hc401", "C13"])

    def test_continue_multiprocess(self):
        class MyResponder(self.BaseResponder):
            pass

        self.server.responder = MyResponder()
        self.runCmd("platform select remote-linux")
        target = self.createTarget("a.yaml")
        process = self.connect(target)
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateExited]
        )
        self.assertPacketLogReceived(["vCont;C13:p400.401"])



"""
Test lldb data formatter subsystem.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class DataFormatterBoolRefPtr(TestBase):
    @skipUnlessDarwin
    def test_boolrefptr_with_run_command(self):
        """Test the formatters we use for BOOL& and BOOL* in Objective-C."""
        self.build()
        self.boolrefptr_data_formatter_commands()

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break at.
        self.line = line_number("main.mm", "// Set break point at this line.")

    def boolrefptr_data_formatter_commands(self):
        """Test the formatters we use for BOOL& and BOOL* in Objective-C."""
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.mm", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type format clear", check=False)
            self.runCmd("type summary clear", check=False)
            self.runCmd("type synth clear", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        isArm = "arm" in self.getArchitecture()

        # Now check that we use the right summary for BOOL&
        self.expect("frame variable yes_ref", substrs=["YES"])
        self.expect("frame variable no_ref", substrs=["NO"])
        if not (isArm):
            self.expect("frame variable unset_ref", substrs=["12"])

        # Now check that we use the right summary for BOOL*
        self.expect("frame variable yes_ptr", substrs=["YES"])
        self.expect("frame variable no_ptr", substrs=["NO"])
        if not (isArm):
            self.expect("frame variable unset_ptr", substrs=["12"])

        # Now check that we use the right summary for BOOL
        self.expect("frame variable yes", substrs=["YES"])
        self.expect("frame variable no", substrs=["NO"])
        if not (isArm):
            self.expect("frame variable unset", substrs=["12"])

        # BOOL is bool instead of signed char on ARM.
        converted_YES = "-1" if not isArm else "YES"

        self.expect_expr(
            "myField",
            result_type="BoolBitFields",
            result_children=[
                ValueCheck(name="fieldOne", summary="NO"),
                ValueCheck(name="fieldTwo", summary=converted_YES),
                ValueCheck(name="fieldThree", summary="NO"),
                ValueCheck(name="fieldFour", summary="NO"),
                ValueCheck(name="fieldFive", summary=converted_YES),
            ],
        )



"""
Tests that importing ObjC modules in a non-ObjC target doesn't crash LLDB.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.c")
        )

        # Import foundation to get some ObjC types.
        self.expect("expr --lang objc -- @import Foundation")
        # Do something with NSString (which requires special handling when
        # preparing to run in the target). The expression most likely can't
        # be prepared to run in the target but it should at least not crash LLDB.
        self.expect(
            'expr --lang objc -- [NSString stringWithFormat:@"%d", 1];',
            error=True,
            substrs=[
                "Rewriting an Objective-C constant string requires CFStringCreateWithBytes"
            ],
        )



"""
An SBTarget with no arch, call AddModule, SBTarget's arch should be set.
"""

import os
import subprocess
import re
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TargetArchFromModule(TestBase):
    SHARED_BUILD_TESTCASE = False

    @skipIf(
        debug_info=no_match(["dsym"]),
        bugnumber="This test is looking explicitly for a dSYM",
    )
    @skipUnlessDarwin
    @skipIfRemote
    def test_target_arch_init(self):
        self.build()
        aout_exe = self.getBuildArtifact("a.out")
        aout_dsym = self.getBuildArtifact("a.out.dSYM")
        hidden_dir = self.getBuildArtifact("hide.noindex")
        hidden_aout_exe = self.getBuildArtifact("hide.noindex/a.out")
        hidden_aout_dsym = self.getBuildArtifact("hide.noindex/a.out.dSYM")
        dsym_for_uuid = self.getBuildArtifact("dsym-for-uuid.sh")

        # We can hook in our dsym-for-uuid shell script to lldb with
        # this env var instead of requiring a defaults write.
        os.environ["LLDB_APPLE_DSYMFORUUID_EXECUTABLE"] = dsym_for_uuid
        self.addTearDownHook(
            lambda: os.environ.pop("LLDB_APPLE_DSYMFORUUID_EXECUTABLE", None)
        )

        dwarfdump_uuid_regex = re.compile(r"UUID: ([-0-9a-fA-F]+) \(([^\(]+)\) .*")
        dwarfdump_cmd_output = subprocess.check_output(
            ('/usr/bin/dwarfdump --uuid "%s"' % aout_exe), shell=True
        ).decode("utf-8")
        aout_uuid = None
        for line in dwarfdump_cmd_output.splitlines():
            match = dwarfdump_uuid_regex.search(line)
            if match:
                aout_uuid = match.group(1)
        self.assertNotEqual(aout_uuid, None, "Could not get uuid of built a.out")

        ###  Create our dsym-for-uuid shell script which returns self.hidden_aout_exe.
        shell_cmds = [
            "#! /bin/sh",
            "# the last argument is the uuid",
            "while [ $# -gt 1 ]",
            "do",
            "  shift",
            "done",
            "ret=0",
            'echo "<?xml version=\\"1.0\\" encoding=\\"UTF-8\\"?>"',
            'echo "<!DOCTYPE plist PUBLIC \\"-//Apple//DTD PLIST 1.0//EN\\" \\"http://www.apple.com/DTDs/PropertyList-1.0.dtd\\">"',
            'echo "<plist version=\\"1.0\\">"',
            "",
            'if [ "$1" = "%s" ]' % aout_uuid,
            "then",
            "  uuid=%s" % aout_uuid,
            "  bin=%s" % hidden_aout_exe,
            "  dsym=%s.dSYM/Contents/Resources/DWARF/%s"
            % (hidden_aout_exe, os.path.basename(hidden_aout_exe)),
            "fi",
            'echo "  <dict>"',
            'echo "    <key>$1</key>"',
            'echo "    <dict>"',
            'if [ -z "$uuid" -o -z "$bin" -o ! -f "$bin" ]',
            "then",
            '  echo "      <key>DBGError</key>"',
            '  echo "      <string>not found by $0</string>"',
            '  echo "    </dict>"',
            '  echo "  </dict>"',
            '  echo "</plist>"',
            "  exit 0",
            "fi",
            "",
            'echo "<key>DBGDSYMPath</key><string>$dsym</string>"',
            'echo "<key>DBGSymbolRichExecutable</key><string>$bin</string>"',
            'echo "</dict></dict></plist>"',
            "exit $ret",
        ]

        with open(dsym_for_uuid, "w") as writer:
            for l in shell_cmds:
                writer.write(l + "\n")

        os.chmod(dsym_for_uuid, 0o755)

        # Move the main binary and its dSYM into the hide.noindex
        # directory.  Now the only way lldb can find them is with
        # the LLDB_APPLE_DSYMFORUUID_EXECUTABLE shell script -
        # so we're testing that this dSYM discovery method works.
        lldbutil.mkdir_p(hidden_dir)
        os.rename(aout_exe, hidden_aout_exe)
        os.rename(aout_dsym, hidden_aout_dsym)

        target = self.dbg.CreateTarget("")
        self.assertTrue(target.IsValid())
        self.expect("target list", matching=False, substrs=["arch="])

        m = target.AddModule(None, None, aout_uuid)
        self.assertTrue(m.IsValid())
        self.expect("target list", substrs=["arch="])



"""
Test lldb-dap setBreakpoints request
"""

import dap_server
import lldbdap_testcase
from lldbsuite.test import lldbutil
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


def get_subprocess(root_process, process_name):
    queue = [root_process]
    while queue:
        process = queue.pop()
        if process.name() == process_name:
            return process
        queue.extend(process.children())

    self.assertTrue(False, "No subprocess with name %s found" % process_name)


class TestDAP_console(lldbdap_testcase.DAPTestCaseBase):
    def check_lldb_command(
        self, lldb_command, contains_string, assert_msg, command_escape_prefix="`"
    ):
        response = self.dap_server.request_evaluate(
            f"{command_escape_prefix}{lldb_command}", context="repl"
        )
        output = response["body"]["result"]
        self.assertIn(
            contains_string,
            output,
            (
                """Verify %s by checking the command output:\n"""
                """'''\n%s'''\nfor the string: "%s" """
                % (assert_msg, output, contains_string)
            ),
        )

    def test_scopes_variables_setVariable_evaluate(self):
        """
        Tests that the "scopes" request causes the currently selected
        thread and frame to be updated. There are no DAP packets that tell
        lldb-dap which thread and frame are selected other than the
        "scopes" request. lldb-dap will now select the thread and frame
        for the latest "scopes" request that it receives.

        The LLDB command interpreter needs to have the right thread and
        frame selected so that commands executed in the debug console act
        on the right scope. This applies both to the expressions that are
        evaluated and the lldb commands that start with the backtick
        character.
        """
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)
        source = "main.cpp"
        breakpoint1_line = line_number(source, "// breakpoint 1")
        lines = [breakpoint1_line]
        # Set breakpoint in the thread function so we can step the threads
        breakpoint_ids = self.set_source_breakpoints(source, lines)
        self.assertEqual(
            len(breakpoint_ids), len(lines), "expect correct number of breakpoints"
        )
        self.continue_to_breakpoints(breakpoint_ids)
        # Cause a "scopes" to be sent for frame zero which should update the
        # selected thread and frame to frame 0.
        self.dap_server.get_local_variables(frameIndex=0)

        # Verify frame #0 is selected in the command interpreter by running
        # the "frame select" command with no frame index which will print the
        # currently selected frame.
        self.check_lldb_command("frame select", "frame #0", "frame 0 is selected")

        # Cause a "scopes" to be sent for frame one which should update the
        # selected thread and frame to frame 1.
        self.dap_server.get_local_variables(frameIndex=1)

        # Verify frame #1 is selected in the command interpreter by running
        # the "frame select" command with no frame index which will print the
        # currently selected frame.
        self.check_lldb_command("frame select", "frame #1", "frame 1 is selected")

    def test_custom_escape_prefix(self):
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program, commandEscapePrefix="::")
        source = "main.cpp"
        breakpoint1_line = line_number(source, "// breakpoint 1")
        breakpoint_ids = self.set_source_breakpoints(source, [breakpoint1_line])
        self.continue_to_breakpoints(breakpoint_ids)

        self.check_lldb_command(
            "help",
            "For more information on any command",
            "Help can be invoked",
            command_escape_prefix="::",
        )

    def test_empty_escape_prefix(self):
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program, commandEscapePrefix="")
        source = "main.cpp"
        breakpoint1_line = line_number(source, "// breakpoint 1")
        breakpoint_ids = self.set_source_breakpoints(source, [breakpoint1_line])
        self.continue_to_breakpoints(breakpoint_ids)

        self.check_lldb_command(
            "help",
            "For more information on any command",
            "Help can be invoked",
            command_escape_prefix="",
        )

    @skipIfWindows
    def test_exit_status_message_sigterm(self):
        source = "main.cpp"
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program, commandEscapePrefix="")
        breakpoint1_line = line_number(source, "// breakpoint 1")
        breakpoint_ids = self.set_source_breakpoints(source, [breakpoint1_line])
        self.continue_to_breakpoints(breakpoint_ids)

        # Kill lldb-server process.
        process_name = (
            "debugserver" if platform.system() in ["Darwin"] else "lldb-server"
        )

        try:
            import psutil
        except ImportError:
            print(
                "psutil not installed, please install using 'pip install psutil'. "
                "Skipping test_exit_status_message_sigterm test.",
                file=sys.stderr,
            )
            return
        process = get_subprocess(psutil.Process(os.getpid()), process_name)
        process.terminate()
        process.wait()

        # Get the console output
        console_output = self.collect_console(pattern="exited with status")

        # Verify the exit status message is printed.
        self.assertRegex(
            console_output,
            ".*exited with status = -1 .* died with signal SIGTERM.*",
            "Exit status does not contain message 'exited with status'",
        )

    def test_exit_status_message_ok(self):
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program, commandEscapePrefix="")
        self.continue_to_exit()

        # Get the console output
        console_output = self.collect_console(pattern="exited with status")

        # Verify the exit status message is printed.
        self.assertIn(
            "exited with status = 0 (0x00000000)",
            console_output,
            "Exit status does not contain message 'exited with status'",
        )

    def test_diagnositcs(self):
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)

        core = self.getBuildArtifact("minidump.core")
        self.yaml2obj("minidump.yaml", core)
        self.dap_server.request_evaluate(
            f"target create --core  {core}", context="repl"
        )

        diagnostics = self.collect_important(pattern="minidump file")

        self.assertIn(
            "warning: unable to retrieve process ID from minidump file",
            diagnostics,
            "diagnostic found in important output",
        )



import lldb
import sys


class WelcomeCommand(object):
    def __init__(self, debugger, session_dict):
        pass

    def get_short_help(self):
        return (
            "Just a docstring for welcome_impl\nA command that says hello to LLDB users"
        )

    def __call__(self, debugger, args, exe_ctx, result):
        print("Hello " + args + ", welcome to LLDB", file=result)
        return None


class TargetnameCommand(object):
    def __init__(self, debugger, session_dict):
        pass

    def __call__(self, debugger, args, exe_ctx, result):
        target = debugger.GetSelectedTarget()
        file = target.GetExecutable()
        print("Current target " + file.GetFilename(), file=result)
        if args == "fail":
            result.SetError("a test for error in command")

    def get_flags(self):
        return lldb.eCommandRequiresTarget


def print_wait_impl(debugger, args, result, dict):
    result.SetImmediateOutputFile(sys.stdout)
    print("Trying to do long task..", file=result)
    import time

    time.sleep(1)
    print("Still doing long task..", file=result)
    time.sleep(1)
    print("Done; if you saw the delays I am doing OK", file=result)


def check_for_synchro(debugger, args, result, dict):
    if debugger.GetAsync():
        print("I am running async", file=result)
    if not debugger.GetAsync():
        print("I am running sync", file=result)


def takes_exe_ctx(debugger, args, exe_ctx, result, dict):
    print(str(exe_ctx.GetTarget()), file=result)



"""
Check that lldb features work when the AArch64 Guarded Control Stack (GCS)
extension is enabled.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class AArch64LinuxGCSTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipUnlessArch("aarch64")
    @skipUnlessPlatform(["linux"])
    def test_gcs_region(self):
        if not self.isAArch64GCS():
            self.skipTest("Target must support GCS.")

        # This test assumes that we have /proc/<PID>/smaps files
        # that include "VmFlags:" lines.
        # AArch64 kernel config defaults to enabling smaps with
        # PROC_PAGE_MONITOR and "VmFlags" was added in kernel 3.8,
        # before GCS was supported at all.

        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self,
            "main.c",
            line_number("main.c", "// Set break point at this line."),
            num_expected_locations=1,
        )

        self.runCmd("run", RUN_SUCCEEDED)

        if self.process().GetState() == lldb.eStateExited:
            self.fail("Test program failed to run.")

        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # By now either the program or the system C library enabled GCS and there
        # should be one region marked for use by it (we cannot predict exactly
        # where it will be).
        self.runCmd("memory region --all")
        found_ss = False
        for line in self.res.GetOutput().splitlines():
            if line.strip() == "shadow stack: yes":
                if found_ss:
                    self.fail("Found more than one shadow stack region.")
                found_ss = True

        self.assertTrue(found_ss, "Failed to find a shadow stack region.")

        # Note that we must let the debugee get killed here as it cannot exit
        # cleanly if GCS was manually enabled.

    @skipUnlessArch("aarch64")
    @skipUnlessPlatform(["linux"])
    def test_gcs_fault(self):
        if not self.isAArch64GCS():
            self.skipTest("Target must support GCS.")

        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)
        self.runCmd("run", RUN_SUCCEEDED)

        if self.process().GetState() == lldb.eStateExited:
            self.fail("Test program failed to run.")

        self.expect(
            "thread list",
            "Expected stopped by SIGSEGV.",
            substrs=[
                "stopped",
                "stop reason = signal SIGSEGV: control protection fault",
            ],
        )

    # This helper reads all the GCS registers and optionally compares them
    # against a previous state, then returns the current register values.
    def check_gcs_registers(
        self,
        expected_gcs_features_enabled=None,
        expected_gcs_features_locked=None,
        expected_gcspr_el0=None,
    ):
        thread = self.dbg.GetSelectedTarget().process.GetThreadAtIndex(0)
        registerSets = thread.GetFrameAtIndex(0).GetRegisters()
        gcs_registers = registerSets.GetFirstValueByName(
            r"Guarded Control Stack Registers"
        )

        gcs_features_enabled = gcs_registers.GetChildMemberWithName(
            "gcs_features_enabled"
        ).GetValueAsUnsigned()
        if expected_gcs_features_enabled is not None:
            self.assertEqual(expected_gcs_features_enabled, gcs_features_enabled)

        gcs_features_locked = gcs_registers.GetChildMemberWithName(
            "gcs_features_locked"
        ).GetValueAsUnsigned()
        if expected_gcs_features_locked is not None:
            self.assertEqual(expected_gcs_features_locked, gcs_features_locked)

        gcspr_el0 = gcs_registers.GetChildMemberWithName(
            "gcspr_el0"
        ).GetValueAsUnsigned()
        if expected_gcspr_el0 is not None:
            self.assertEqual(expected_gcspr_el0, gcspr_el0)

        return gcs_features_enabled, gcs_features_locked, gcspr_el0

    @skipUnlessArch("aarch64")
    @skipUnlessPlatform(["linux"])
    def test_gcs_registers(self):
        if not self.isAArch64GCS():
            self.skipTest("Target must support GCS.")

        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        self.runCmd("b test_func")
        self.runCmd("b test_func2")
        self.runCmd("run", RUN_SUCCEEDED)

        if self.process().GetState() == lldb.eStateExited:
            self.fail("Test program failed to run.")

        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        self.expect("register read --all", substrs=["Guarded Control Stack Registers:"])

        enabled, locked, spr_el0 = self.check_gcs_registers()

        # Features enabled should have at least the enable bit set, it could have
        # others depending on what the C library did, but we can't rely on always
        # having them.
        self.assertTrue(enabled & 1, "Expected GCS enable bit to be set.")

        # Features locked we cannot predict, we will just assert that it remains
        # the same as we continue.

        # spr_el0 will point to some memory region that is a shadow stack region.
        self.expect(f"memory region {spr_el0}", substrs=["shadow stack: yes"])

        # Continue into test_func2, where the GCS pointer should have been
        # decremented, and the other registers remain the same.
        self.runCmd("continue")

        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        _, _, spr_el0 = self.check_gcs_registers(enabled, locked, spr_el0 - 8)

        # Any combination of GCS feature lock bits might have been set by the C
        # library, and could be set to 0 or 1. To check that we can modify them,
        # invert one of those bits then write it back to the lock register.
        # The stack pushing feature is bit 2 of that register.
        STACK_PUSH = 2
        # Get the original value of the stack push lock bit.
        stack_push = bool((locked >> STACK_PUSH) & 1)
        # Invert the value and put it back into the set of lock bits.
        new_locked = (locked & ~(1 << STACK_PUSH)) | (int(not stack_push) << STACK_PUSH)
        # Write the new lock bits, which are the same as before, only with stack
        # push locked (if it was previously unlocked), or unlocked (if it was
        # previously locked).
        self.runCmd(f"register write gcs_features_locked 0x{new_locked:x}")
        # We should be able to read back this new set of lock bits.
        self.expect(
            f"register read gcs_features_locked",
            substrs=[f"gcs_features_locked = 0x{new_locked:016x}"],
        )

        # We could prove the write made it to hardware by trying to prctl() to
        # enable or disable the stack push feature here, but because the libc
        # may or may not have locked it, it's tricky to coordinate this. Given
        # that we know the other registers can be written and their values are
        # seen by the process, we can assume this is too.

        # Restore the original lock bits, as the libc may rely on being able
        # to use certain features during program execution.
        self.runCmd(f"register write gcs_features_locked 0x{locked:x}")

        # Modify the guarded control stack pointer to cause a fault.
        spr_el0 += 8
        self.runCmd(f"register write gcspr_el0 {spr_el0}")
        self.expect(
            "register read gcspr_el0", substrs=[f"gcspr_el0 = 0x{spr_el0:016x}"]
        )

        # If we wrote it back correctly, we will now fault. Don't pass this signal
        # to the application, as we will continue past it later.
        self.runCmd("process handle SIGSEGV --pass false")
        self.runCmd("continue")

        self.expect(
            "thread list",
            "Expected stopped by SIGSEGV.",
            substrs=[
                "stopped",
                "stop reason = signal SIGSEGV: control protection fault",
            ],
        )

        # Now to prove we can write gcs_features_enabled, disable GCS and continue
        # past the fault we caused. Note that although the libc likely locked the
        # ability to disable GCS, ptrace bypasses the lock bits.
        enabled &= ~1
        self.runCmd(f"register write gcs_features_enabled {enabled}")
        self.expect(
            "register read gcs_features_enabled",
            substrs=[
                f"gcs_features_enabled = 0x{enabled:016x}",
                f"= (PUSH = {(enabled >> 2) & 1}, WRITE = {(enabled >> 1) & 1}, ENABLE = {enabled & 1})",
            ],
        )

        # With GCS disabled, the invalid guarded control stack pointer is not
        # checked, so the program can finish normally.
        self.runCmd("continue")
        self.expect(
            "process status",
            substrs=[
                "exited with status = 0",
            ],
        )

    @skipUnlessPlatform(["linux"])
    def test_gcs_expression_simple(self):
        if not self.isAArch64GCS():
            self.skipTest("Target must support GCS.")

        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        # Break before GCS has been enabled.
        self.runCmd("b main")
        # And after it has been enabled.
        lldbutil.run_break_set_by_file_and_line(
            self,
            "main.c",
            line_number("main.c", "// Set break point at this line."),
            num_expected_locations=1,
        )

        self.runCmd("run", RUN_SUCCEEDED)

        if self.process().GetState() == lldb.eStateExited:
            self.fail("Test program failed to run.")

        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # GCS has not been enabled yet and the ABI plugin should know not to
        # attempt pushing to the control stack.
        before = self.check_gcs_registers()
        expr_cmd = "p get_gcs_status()"
        self.expect(expr_cmd, substrs=["(unsigned long) 0"])
        self.check_gcs_registers(*before)

        # Continue to when GCS has been enabled.
        self.runCmd("continue")
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # If we fail to setup the GCS entry, we should not leave any of the GCS registers
        # changed. The last thing we do is write a new GCS entry to memory and
        # to simulate the failure of that, temporarily point the GCS to the zero page.
        #
        # We use the value 8 here because LLDB will decrement it by 8 so it points to
        # what we think will be an empty entry on the guarded control stack.
        _, _, original_gcspr = self.check_gcs_registers()
        self.runCmd("register write gcspr_el0 8")
        before = self.check_gcs_registers()
        self.expect(expr_cmd, error=True)
        self.check_gcs_registers(*before)
        # Point to the valid shadow stack region again.
        self.runCmd(f"register write gcspr_el0 {original_gcspr}")

        # This time we do need to push to the GCS and having done so, we can
        # return from this expression without causing a fault.
        before = self.check_gcs_registers()
        self.expect(expr_cmd, substrs=["(unsigned long) 1"])
        self.check_gcs_registers(*before)

    @skipUnlessPlatform(["linux"])
    def test_gcs_expression_disable_gcs(self):
        if not self.isAArch64GCS():
            self.skipTest("Target must support GCS.")

        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        # Break after GCS is enabled.
        lldbutil.run_break_set_by_file_and_line(
            self,
            "main.c",
            line_number("main.c", "// Set break point at this line."),
            num_expected_locations=1,
        )

        self.runCmd("run", RUN_SUCCEEDED)

        if self.process().GetState() == lldb.eStateExited:
            self.fail("Test program failed to run.")

        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # Unlock all features so the expression can enable them again.
        self.runCmd("register write gcs_features_locked 0")
        # Disable all features, but keep GCS itself enabled.
        PR_SHADOW_STACK_ENABLE = 1
        self.runCmd(f"register write gcs_features_enabled 0x{PR_SHADOW_STACK_ENABLE:x}")

        enabled, locked, spr_el0 = self.check_gcs_registers()
        # We restore everything apart GCS being enabled, as we are not allowed to
        # go from disabled -> enabled via ptrace.
        self.expect("p change_gcs_config(false)", substrs=["true"])
        enabled &= ~1
        self.check_gcs_registers(enabled, locked, spr_el0)

    @skipUnlessPlatform(["linux"])
    def test_gcs_expression_enable_gcs(self):
        if not self.isAArch64GCS():
            self.skipTest("Target must support GCS.")

        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        # Break before GCS is enabled.
        self.runCmd("b main")

        self.runCmd("run", RUN_SUCCEEDED)

        if self.process().GetState() == lldb.eStateExited:
            self.fail("Test program failed to run.")

        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # Unlock all features so the expression can enable them again.
        self.runCmd("register write gcs_features_locked 0")
        # Disable all features. The program needs PR_SHADOW_STACK_PUSH, but it
        # will enable that itself.
        self.runCmd(f"register write gcs_features_enabled 0")

        enabled, locked, spr_el0 = self.check_gcs_registers()
        self.expect("p change_gcs_config(true)", substrs=["true"])
        # Though we could disable GCS with ptrace, we choose not to to be
        # consistent with the disabled -> enabled behaviour.
        enabled |= 1
        self.check_gcs_registers(enabled, locked, spr_el0)

    @skipIfLLVMTargetMissing("AArch64")
    def test_gcs_core_file(self):
        # To re-generate the core file, build the test file and run it on a
        # machine with GCS enabled. Note that because the kernel decides where
        # the GCS is stored, the value of gcspr_el0 and which memory region it
        # points to may change between runs.

        self.runCmd("target create --core corefile")

        self.expect(
            "bt",
            substrs=["stop reason = SIGSEGV: control protection fault"],
        )

        self.expect(
            "register read --all",
            substrs=[
                "Guarded Control Stack Registers:",
                "gcs_features_enabled = 0x0000000000000001",
                "gcs_features_locked = 0x0000000000000000",
                "gcspr_el0 = 0x0000ffffa83ffff0",
            ],
        )

        # Should get register fields for both. They have the same fields.
        self.expect(
            "register read gcs_features_enabled",
            substrs=["= (PUSH = 0, WRITE = 0, ENABLE = 1)"],
        )
        self.expect(
            "register read gcs_features_locked",
            substrs=["= (PUSH = 0, WRITE = 0, ENABLE = 0)"],
        )

        # Core files do not include /proc/pid/smaps, so we cannot see the
        # shadow stack "ss" flag. gcspr_el0 should at least point to some mapped
        # region.
        self.expect(
            "memory region $gcspr_el0",
            substrs=["[0x0000ffffa8000000-0x0000ffffa8400000) rw-"],
        )



from lldbsuite.test.lldbinline import CommandParser
from lldbsuite.test.lldbtest import Base
import textwrap


class TestCommandParser(Base):
    def test_indentation(self):
        """Test indentation handling"""
        filename = self.getBuildArtifact("test_file.cpp")
        with open(filename, "w") as f:
            f.write(
                textwrap.dedent(
                    """\
                    int q;
                    int w; //% first break
                    int e;
                    int r; //% second break
                    //% continue second
                    //%   continuing indented
                      //% not indented
                    int t; //% third break
                    """
                )
            )
        p = CommandParser()
        p.parse_source_files([filename])

        def bkpt(line, cmd):
            return {"file_name": filename, "line_number": line, "command": cmd}

        self.assertEqual(
            p.breakpoints,
            [
                bkpt(2, "first break"),
                bkpt(
                    4,
                    "second break\ncontinue second\n  continuing indented\nnot indented",
                ),
                bkpt(8, "third break"),
            ],
        )



# encoding: utf-8
"""
Test lldb data formatter subsystem.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

from ObjCDataFormatterTestCase import ObjCDataFormatterTestCase


class ObjCDataFormatterNSBundle(ObjCDataFormatterTestCase):
    SHARED_BUILD_TESTCASE = False

    def test_nsbundle_with_run_command(self):
        """Test formatters for NSBundle."""
        self.appkit_tester_impl(self.nsbundle_data_formatter_commands, True)

    @skipUnlessDarwin
    def test_nsbundle_with_run_command_no_sonct(self):
        """Test formatters for NSBundle."""
        self.appkit_tester_impl(self.nsbundle_data_formatter_commands, False)

    def nsbundle_data_formatter_commands(self):
        self.expect(
            "frame variable bundle_string bundle_url main_bundle",
            substrs=[
                "(NSBundle *) bundle_string = ",
                ' @"/System/Library/Frameworks/Accelerate.framework"',
                "(NSBundle *) bundle_url = ",
                ' @"/System/Library/Frameworks/Foundation.framework"',
                "(NSBundle *) main_bundle = ",
                "data-formatter-objc",
            ],
        )



# RUN: %PYTHON %s | FileCheck %s

import gc

from mlir.ir import *


def run(f):
    print("\nTEST:", f.__name__)
    f()
    gc.collect()
    assert Context._get_live_count() == 0
    return f


# CHECK-LABEL: TEST: testParsePrint
@run
def testParsePrint():
    with Context() as ctx:
        t = Attribute.parse('"hello"')
    assert t.context is ctx
    ctx = None
    gc.collect()
    # CHECK: "hello"
    print(str(t))
    # CHECK: StringAttr("hello")
    print(repr(t))


# CHECK-LABEL: TEST: testParseError
@run
def testParseError():
    with Context():
        try:
            t = Attribute.parse("BAD_ATTR_DOES_NOT_EXIST")
        except MLIRError as e:
            # CHECK: testParseError: <
            # CHECK:   Unable to parse attribute:
            # CHECK:   error: "BAD_ATTR_DOES_NOT_EXIST":1:1: expected attribute value
            # CHECK: >
            print(f"testParseError: <{e}>")
        else:
            print("Exception not produced")


# CHECK-LABEL: TEST: testAttrEq
@run
def testAttrEq():
    with Context():
        a1 = Attribute.parse('"attr1"')
        a2 = Attribute.parse('"attr2"')
        a3 = Attribute.parse('"attr1"')
        # CHECK: a1 == a1: True
        print("a1 == a1:", a1 == a1)
        # CHECK: a1 == a2: False
        print("a1 == a2:", a1 == a2)
        # CHECK: a1 == a3: True
        print("a1 == a3:", a1 == a3)
        # CHECK: a1 is None: False
        print("a1 is None:", a1 is None)


# CHECK-LABEL: TEST: testAttrHash
@run
def testAttrHash():
    with Context():
        a1 = Attribute.parse('"attr1"')
        a2 = Attribute.parse('"attr2"')
        a3 = Attribute.parse('"attr1"')
        # CHECK: hash(a1) == hash(a3): True
        print("hash(a1) == hash(a3):", a1.__hash__() == a3.__hash__())

        s = set()
        s.add(a1)
        s.add(a2)
        s.add(a3)
        # CHECK: len(s): 2
        print("len(s): ", len(s))


# CHECK-LABEL: TEST: testAttrCast
@run
def testAttrCast():
    with Context():
        a1 = Attribute.parse('"attr1"')
        a2 = Attribute(a1)
        # CHECK: a1 == a2: True
        print("a1 == a2:", a1 == a2)


# CHECK-LABEL: TEST: testAttrIsInstance
@run
def testAttrIsInstance():
    with Context():
        a1 = Attribute.parse("42")
        a2 = Attribute.parse("[42]")
        assert isinstance(a1, IntegerAttr)
        assert not isinstance(a2, IntegerAttr)
        assert not isinstance(a1, ArrayAttr)
        assert isinstance(a2, ArrayAttr)


# CHECK-LABEL: TEST: testAttrEqDoesNotRaise
@run
def testAttrEqDoesNotRaise():
    with Context():
        a1 = Attribute.parse('"attr1"')
        not_an_attr = "foo"
        # CHECK: False
        print(a1 == not_an_attr)
        # CHECK: False
        print(a1 is None)
        # CHECK: True
        print(a1 is not None)


# CHECK-LABEL: TEST: testAttrCapsule
@run
def testAttrCapsule():
    with Context() as ctx:
        a1 = Attribute.parse('"attr1"')
    # CHECK: mlir.ir.Attribute._CAPIPtr
    attr_capsule = a1._CAPIPtr
    print(attr_capsule)
    a2 = Attribute._CAPICreate(attr_capsule)
    assert a2 == a1
    assert a2.context is ctx


# CHECK-LABEL: TEST: testStandardAttrCasts
@run
def testStandardAttrCasts():
    with Context():
        a1 = Attribute.parse('"attr1"')
        astr = StringAttr(a1)
        aself = StringAttr(astr)
        # CHECK: StringAttr("attr1")
        print(repr(astr))
        try:
            tillegal = StringAttr(Attribute.parse("1.0"))
        except ValueError as e:
            # CHECK: ValueError: Cannot cast attribute to StringAttr (from Attribute(1.000000e+00 : f64))
            print("ValueError:", e)
        else:
            print("Exception not produced")


# CHECK-LABEL: TEST: testAffineMapAttr
@run
def testAffineMapAttr():
    with Context() as ctx:
        d0 = AffineDimExpr.get(0)
        d1 = AffineDimExpr.get(1)
        c2 = AffineConstantExpr.get(2)
        map0 = AffineMap.get(2, 3, [])

        # CHECK: affine_map<(d0, d1)[s0, s1, s2] -> ()>
        attr_built = AffineMapAttr.get(map0)
        print(str(attr_built))
        assert attr_built.value == map0
        attr_parsed = Attribute.parse(str(attr_built))
        assert attr_built == attr_parsed


# CHECK-LABEL: TEST: testIntegerSetAttr
@run
def testIntegerSetAttr():
    with Context() as ctx:
        d0 = AffineDimExpr.get(0)
        d1 = AffineDimExpr.get(1)
        s0 = AffineSymbolExpr.get(0)
        c42 = AffineConstantExpr.get(42)
        set0 = IntegerSet.get(2, 1, [d0 - d1, s0 - c42], [True, False])

        # CHECK: affine_set<(d0, d1)[s0] : (d0 - d1 == 0, s0 - 42 >= 0)>
        attr_built = IntegerSetAttr.get(set0)
        print(str(attr_built))

        attr_parsed = Attribute.parse(str(attr_built))
        assert attr_built == attr_parsed


# CHECK-LABEL: TEST: testFloatAttr
@run
def testFloatAttr():
    with Context(), Location.unknown():
        fattr = FloatAttr(Attribute.parse("42.0 : f32"))
        # CHECK: fattr value: 42.0
        print("fattr value:", fattr.value)
        # CHECK: fattr float: 42.0 <class 'float'>
        print("fattr float:", float(fattr), type(float(fattr)))

        # Test factory methods.
        # CHECK: default_get: 4.200000e+01 : f32
        print("default_get:", FloatAttr.get(F32Type.get(), 42.0))
        # CHECK: f32_get: 4.200000e+01 : f32
        print("f32_get:", FloatAttr.get_f32(42.0))
        # CHECK: f64_get: 4.200000e+01 : f64
        print("f64_get:", FloatAttr.get_f64(42.0))
        try:
            fattr_invalid = FloatAttr.get(IntegerType.get_signless(32), 42)
        except MLIRError as e:
            # CHECK: Invalid attribute:
            # CHECK: error: unknown: expected floating point type
            print(e)
        else:
            print("Exception not produced")


# CHECK-LABEL: TEST: testIntegerAttr
@run
def testIntegerAttr():
    with Context() as ctx:
        i_attr = IntegerAttr(Attribute.parse("42"))
        # CHECK: i_attr value: 42
        print("i_attr value:", i_attr.value)
        # CHECK: i_attr type: i64
        print("i_attr type:", i_attr.type)
        # CHECK: i_attr int: 42 <class 'int'>
        print("i_attr int:", int(i_attr), type(int(i_attr)))
        si_attr = IntegerAttr(Attribute.parse("-1 : si8"))
        # CHECK: si_attr value: -1
        print("si_attr value:", si_attr.value)
        ui_attr = IntegerAttr(Attribute.parse("255 : ui8"))
        # CHECK: i_attr int: -1 <class 'int'>
        print("si_attr int:", int(si_attr), type(int(si_attr)))
        # CHECK: ui_attr value: 255
        print("ui_attr value:", ui_attr.value)
        # CHECK: i_attr int: 255 <class 'int'>
        print("ui_attr int:", int(ui_attr), type(int(ui_attr)))
        idx_attr = IntegerAttr(Attribute.parse("-1 : index"))
        # CHECK: idx_attr value: -1
        print("idx_attr value:", idx_attr.value)
        # CHECK: idx_attr int: -1 <class 'int'>
        print("idx_attr int:", int(idx_attr), type(int(idx_attr)))

        # Test factory methods.
        # CHECK: default_get: 42 : i32
        print("default_get:", IntegerAttr.get(IntegerType.get_signless(32), 42))


# CHECK-LABEL: TEST: testBoolAttr
@run
def testBoolAttr():
    with Context() as ctx:
        battr = BoolAttr(Attribute.parse("true"))
        # CHECK: iattr value: True
        print("iattr value:", battr.value)
        # CHECK: iattr bool: True <class 'bool'>
        print("iattr bool:", bool(battr), type(bool(battr)))

        # Test factory methods.
        # CHECK: default_get: true
        print("default_get:", BoolAttr.get(True))


# CHECK-LABEL: TEST: testFlatSymbolRefAttr
@run
def testFlatSymbolRefAttr():
    with Context() as ctx:
        sattr = Attribute.parse("@symbol")
        # CHECK: symattr value: symbol
        print("symattr value:", sattr.value)

        # Test factory methods.
        # CHECK: default_get: @foobar
        print("default_get:", FlatSymbolRefAttr.get("foobar"))


# CHECK-LABEL: TEST: testSymbolRefAttr
@run
def testSymbolRefAttr():
    with Context() as ctx:
        sattr = Attribute.parse("@symbol1::@symbol2")
        # CHECK: symattr value: ['symbol1', 'symbol2']
        print("symattr value:", sattr.value)

        # CHECK: default_get: @symbol1::@symbol2
        print("default_get:", SymbolRefAttr.get(["symbol1", "symbol2"]))

        # CHECK: default_get: @"@symbol1"::@"@symbol2"
        print("default_get:", SymbolRefAttr.get(["@symbol1", "@symbol2"]))


# CHECK-LABEL: TEST: testOpaqueAttr
@run
def testOpaqueAttr():
    with Context() as ctx:
        ctx.allow_unregistered_dialects = True
        oattr = OpaqueAttr(Attribute.parse("#pytest_dummy.dummyattr<>"))
        # CHECK: oattr value: pytest_dummy
        print("oattr value:", oattr.dialect_namespace)
        # CHECK: oattr value: b'dummyattr<>'
        print("oattr value:", oattr.data)

        # Test factory methods.
        # CHECK: default_get: #foobar<123>
        print(
            "default_get:",
            OpaqueAttr.get("foobar", bytes("123", "utf-8"), NoneType.get()),
        )


# CHECK-LABEL: TEST: testStringAttr
@run
def testStringAttr():
    with Context() as ctx:
        sattr = StringAttr(Attribute.parse('"stringattr"'))
        # CHECK: sattr value: stringattr
        print("sattr value:", sattr.value)
        # CHECK: sattr value: b'stringattr'
        print("sattr value:", sattr.value_bytes)

        # Test factory methods.
        # CHECK: default_get: "foobar"
        print("default_get:", StringAttr.get("foobar"))
        # CHECK: typed_get: "12345" : i32
        print("typed_get:", StringAttr.get_typed(IntegerType.get_signless(32), "12345"))


# CHECK-LABEL: TEST: testNamedAttr
@run
def testNamedAttr():
    with Context():
        a = Attribute.parse('"stringattr"')
        named = a.get_named("foobar")  # Note: under the small object threshold
        # CHECK: attr: "stringattr"
        print("attr:", named.attr)
        # CHECK: name: foobar
        print("name:", named.name)
        # CHECK: named: NamedAttribute(foobar="stringattr")
        print("named:", named)


# CHECK-LABEL: TEST: testDenseIntAttr
@run
def testDenseIntAttr():
    with Context():
        raw = Attribute.parse("dense<[[0,1,2],[3,4,5]]> : vector<2x3xi32>")
        # CHECK: attr: dense<[{{\[}}0, 1, 2], [3, 4, 5]]>
        print("attr:", raw)

        a = DenseIntElementsAttr(raw)
        assert len(a) == 6

        # CHECK: 0 1 2 3 4 5
        for value in a:
            print(value, end=" ")
        print()

        # CHECK: i32
        print(ShapedType(a.type).element_type)

        raw = Attribute.parse("dense<[true,false,true,false]> : vector<4xi1>")
        # CHECK: attr: dense<[true, false, true, false]>
        print("attr:", raw)

        a = DenseIntElementsAttr(raw)
        assert len(a) == 4

        # CHECK: 1 0 1 0
        for value in a:
            print(value, end=" ")
        print()

        # CHECK: i1
        print(ShapedType(a.type).element_type)

        shape = Attribute.parse("dense<[0, 1, 2, 3]> : vector<4xindex>")
        # CHECK: attr: dense<[0, 1, 2, 3]>
        print("attr:", shape)


@run
def testDenseArrayGetItem():
    def print_item(attr_asm):
        attr = Attribute.parse(attr_asm)
        print(f"{len(attr)}: {attr[0]}, {attr[1]}")

    with Context():
        # CHECK: 2: False, True
        print_item("array<i1: false, true>")
        # CHECK: 2: 2, 3
        print_item("array<i8: 2, 3>")
        # CHECK: 2: 4, 5
        print_item("array<i16: 4, 5>")
        # CHECK: 2: 6, 7
        print_item("array<i32: 6, 7>")
        # CHECK: 2: 8, 9
        print_item("array<i64: 8, 9>")
        # CHECK: 2: 1.{{0+}}, 2.{{0+}}
        print_item("array<f32: 1.0, 2.0>")
        # CHECK: 2: 3.{{0+}}, 4.{{0+}}
        print_item("array<f64: 3.0, 4.0>")

        class MyBool:
            def __bool__(self):
                return True

        # CHECK: myboolarray: array<i1: true>
        print("myboolarray:", DenseBoolArrayAttr.get([MyBool()]))


# CHECK-LABEL: TEST: testDenseArrayAttrConstruction
@run
def testDenseArrayAttrConstruction():
    with Context(), Location.unknown():

        def create_and_print(cls, x):
            try:
                darr = cls.get(x)
                print(f"input: {x} ({type(x)}), result: {darr}")
            except Exception as ex:
                print(f"input: {x} ({type(x)}), error: {ex}")

        # CHECK: input: [4, 2] (<class 'list'>),
        # CHECK-SAME: result: array<i8: 4, 2>
        create_and_print(DenseI8ArrayAttr, [4, 2])

        # CHECK: input: [4, 2.0] (<class 'list'>),
        # CHECK-SAME: error: get(): incompatible function arguments
        create_and_print(DenseI8ArrayAttr, [4, 2.0])

        # CHECK: input: [40000, 2] (<class 'list'>),
        # CHECK-SAME: error: get(): incompatible function arguments
        create_and_print(DenseI8ArrayAttr, [40000, 2])

        # CHECK: input: range(0, 4) (<class 'range'>),
        # CHECK-SAME: result: array<i8: 0, 1, 2, 3>
        create_and_print(DenseI8ArrayAttr, range(4))

        # CHECK: input: [IntegerAttr(4 : i64), IntegerAttr(2 : i64)] (<class 'list'>),
        # CHECK-SAME: result: array<i8: 4, 2>
        create_and_print(DenseI8ArrayAttr, [Attribute.parse(f"{x}") for x in [4, 2]])

        # CHECK: input: [IntegerAttr(4000 : i64), IntegerAttr(2 : i64)] (<class 'list'>),
        # CHECK-SAME: error: get(): incompatible function arguments
        create_and_print(DenseI8ArrayAttr, [Attribute.parse(f"{x}") for x in [4000, 2]])

        # CHECK: input: [IntegerAttr(4 : i64), FloatAttr(2.000000e+00 : f64)] (<class 'list'>),
        # CHECK-SAME: error: get(): incompatible function arguments
        create_and_print(DenseI8ArrayAttr, [Attribute.parse(f"{x}") for x in [4, 2.0]])

        # CHECK: input: [IntegerAttr(4 : i8), IntegerAttr(2 : ui16)] (<class 'list'>),
        # CHECK-SAME: result: array<i8: 4, 2>
        create_and_print(
            DenseI8ArrayAttr, [Attribute.parse(s) for s in ["4 : i8", "2 : ui16"]]
        )

        # CHECK: input: [FloatAttr(4.000000e+00 : f64), FloatAttr(2.000000e+00 : f64)] (<class 'list'>)
        # CHECK-SAME: result: array<f32: 4.000000e+00, 2.000000e+00>
        create_and_print(
            DenseF32ArrayAttr, [Attribute.parse(f"{x}") for x in [4.0, 2.0]]
        )

        # CHECK: [BoolAttr(true), BoolAttr(false)] (<class 'list'>),
        # CHECK-SAME: result: array<i1: true, false>
        create_and_print(
            DenseBoolArrayAttr, [Attribute.parse(f"{x}") for x in ["true", "false"]]
        )


# CHECK-LABEL: TEST: testDenseIntAttrGetItem
@run
def testDenseIntAttrGetItem():
    def print_item(attr_asm):
        attr = Attribute.parse(attr_asm)
        dtype = ShapedType(attr.type).element_type
        try:
            item = attr[0]
            print(f"{dtype}:", item)
        except TypeError as e:
            print(f"{dtype}:", e)

    with Context():
        # CHECK: i1: 1
        print_item("dense<true> : tensor<i1>")
        # CHECK: i8: 123
        print_item("dense<123> : tensor<i8>")
        # CHECK: i16: 123
        print_item("dense<123> : tensor<i16>")
        # CHECK: i32: 123
        print_item("dense<123> : tensor<i32>")
        # CHECK: i64: 123
        print_item("dense<123> : tensor<i64>")
        # CHECK: ui8: 123
        print_item("dense<123> : tensor<ui8>")
        # CHECK: ui16: 123
        print_item("dense<123> : tensor<ui16>")
        # CHECK: ui32: 123
        print_item("dense<123> : tensor<ui32>")
        # CHECK: ui64: 123
        print_item("dense<123> : tensor<ui64>")
        # CHECK: si8: -123
        print_item("dense<-123> : tensor<si8>")
        # CHECK: si16: -123
        print_item("dense<-123> : tensor<si16>")
        # CHECK: si32: -123
        print_item("dense<-123> : tensor<si32>")
        # CHECK: si64: -123
        print_item("dense<-123> : tensor<si64>")

        # CHECK: i7: Unsupported integer type
        print_item("dense<123> : tensor<i7>")


# CHECK-LABEL: TEST: testDenseFPAttr
@run
def testDenseFPAttr():
    with Context():
        raw = Attribute.parse("dense<[0.0, 1.0, 2.0, 3.0]> : vector<4xf32>")
        # CHECK: attr: dense<[0.000000e+00, 1.000000e+00, 2.000000e+00, 3.000000e+00]>

        print("attr:", raw)

        a = DenseFPElementsAttr(raw)
        assert len(a) == 4

        # CHECK: 0.0 1.0 2.0 3.0
        for value in a:
            print(value, end=" ")
        print()

        # CHECK: f32
        print(ShapedType(a.type).element_type)


# CHECK-LABEL: TEST: testDictAttr
@run
def testDictAttr():
    with Context():
        dict_attr = {
            "stringattr": StringAttr.get("string"),
            "integerattr": IntegerAttr.get(IntegerType.get_signless(32), 42),
        }

        a = DictAttr.get(dict_attr)

        # CHECK: attr: {integerattr = 42 : i32, stringattr = "string"}
        print("attr:", a)

        assert len(a) == 2

        # CHECK: integerattr: IntegerAttr(42 : i32)
        print("integerattr:", repr(a["integerattr"]))

        # CHECK: stringattr: StringAttr("string")
        print("stringattr:", repr(a["stringattr"]))

        # CHECK: True
        print("stringattr" in a)

        # CHECK: False
        print("not_in_dict" in a)

        # Check that exceptions are raised as expected.
        try:
            _ = a["does_not_exist"]
        except KeyError:
            pass
        else:
            assert False, "Exception not produced"

        try:
            _ = a[42]
        except IndexError:
            pass
        else:
            assert False, "expected IndexError on accessing an out-of-bounds attribute"

        # CHECK: empty: {}
        print("empty: ", DictAttr.get())


# CHECK-LABEL: TEST: testTypeAttr
@run
def testTypeAttr():
    with Context():
        raw = Attribute.parse("vector<4xf32>")
        # CHECK: attr: vector<4xf32>
        print("attr:", raw)
        type_attr = TypeAttr(raw)
        # CHECK: f32
        print(ShapedType(type_attr.value).element_type)


# CHECK-LABEL: TEST: testArrayAttr
@run
def testArrayAttr():
    with Context():
        arr = Attribute.parse("[42, true, vector<4xf32>]")
    # CHECK: arr: [42, true, vector<4xf32>]
    print("arr:", arr)
    # CHECK: - IntegerAttr(42 : i64)
    # CHECK: - BoolAttr(true)
    # CHECK: - TypeAttr(vector<4xf32>)
    for attr in arr:
        print("- ", repr(attr))

    with Context():
        intAttr = Attribute.parse("42")
        vecAttr = Attribute.parse("vector<4xf32>")
        boolAttr = BoolAttr.get(True)
        raw = ArrayAttr.get([vecAttr, boolAttr, intAttr])
    # CHECK: attr: [vector<4xf32>, true, 42]
    print("raw attr:", raw)
    # CHECK: - TypeAttr(vector<4xf32>)
    # CHECK: - BoolAttr(true
    # CHECK: - IntegerAttr(42 : i64)
    arr = raw
    for attr in arr:
        print("- ", repr(attr))
    # CHECK: attr[0]: TypeAttr(vector<4xf32>)
    print("attr[0]:", repr(arr[0]))
    # CHECK: attr[1]: BoolAttr(true)
    print("attr[1]:", repr(arr[1]))
    # CHECK: attr[2]: IntegerAttr(42 : i64)
    print("attr[2]:", repr(arr[2]))
    try:
        print("attr[3]:", arr[3])
    except IndexError as e:
        # CHECK: Error: ArrayAttribute index out of range
        print("Error: ", e)
    with Context():
        try:
            ArrayAttr.get([None])
        except RuntimeError as e:
            # CHECK: Error: Invalid attribute (None?) when attempting to create an ArrayAttribute
            print("Error: ", e)
        try:
            ArrayAttr.get([42])
        except RuntimeError as e:
            # CHECK: Error: Invalid attribute when attempting to create an ArrayAttribute
            print("Error: ", e)

    with Context():
        array = ArrayAttr.get([StringAttr.get("a"), StringAttr.get("b")])
        array = array + [StringAttr.get("c")]
        # CHECK: concat: ["a", "b", "c"]
        print("concat: ", array)


# CHECK-LABEL: TEST: testStridedLayoutAttr
@run
def testStridedLayoutAttr():
    with Context():
        attr = StridedLayoutAttr.get(42, [5, 7, 13])
        # CHECK: strided<[5, 7, 13], offset: 42>
        print(attr)
        # CHECK: 42
        print(attr.offset)
        # CHECK: 3
        print(len(attr.strides))
        # CHECK: 5
        print(attr.strides[0])
        # CHECK: 7
        print(attr.strides[1])
        # CHECK: 13
        print(attr.strides[2])

        attr = StridedLayoutAttr.get_fully_dynamic(3)
        dynamic = ShapedType.get_dynamic_stride_or_offset()
        # CHECK: strided<[?, ?, ?], offset: ?>
        print(attr)
        # CHECK: offset is dynamic: True
        print(f"offset is dynamic: {attr.offset == dynamic}")
        # CHECK: rank: 3
        print(f"rank: {len(attr.strides)}")
        # CHECK: strides are dynamic: [True, True, True]
        print(f"strides are dynamic: {[s == dynamic for s in attr.strides]}")


# CHECK-LABEL: TEST: testConcreteTypesRoundTrip
@run
def testConcreteTypesRoundTrip():
    with Context(), Location.unknown():

        def print_item(attr):
            print(repr(attr.type))

        # CHECK: F32Type(f32)
        print_item(Attribute.parse("42.0 : f32"))
        # CHECK: F32Type(f32)
        print_item(FloatAttr.get_f32(42.0))
        # CHECK: IntegerType(i64)
        print_item(IntegerAttr.get(IntegerType.get_signless(64), 42))

        def print_container_item(attr_asm):
            attr = DenseElementsAttr(Attribute.parse(attr_asm))
            print(repr(attr.type))
            print(repr(attr.type.element_type))

        # CHECK: RankedTensorType(tensor<i16>)
        # CHECK: IntegerType(i16)
        print_container_item("dense<123> : tensor<i16>")

        # CHECK: RankedTensorType(tensor<f64>)
        # CHECK: F64Type(f64)
        print_container_item("dense<1.0> : tensor<f64>")

        raw = Attribute.parse("vector<4xf32>")
        # CHECK: attr: vector<4xf32>
        print("attr:", raw)
        type_attr = TypeAttr(raw)

        # CHECK: VectorType(vector<4xf32>)
        print(repr(type_attr.value))
        # CHECK: F32Type(f32)
        print(repr(type_attr.value.element_type))


# CHECK-LABEL: TEST: testConcreteAttributesRoundTrip
@run
def testConcreteAttributesRoundTrip():
    with Context(), Location.unknown():
        # CHECK: FloatAttr(4.200000e+01 : f32)
        print(repr(Attribute.parse("42.0 : f32")))

        assert IntegerAttr.static_typeid is not None


# CHECK-LABEL: TEST: testAttrNames
@run
def testAttrNames():
    with Context():
        # CHECK: builtin.affine_map
        print(AffineMapAttr.attr_name)
        # CHECK: builtin.integer_set
        print(IntegerSetAttr.attr_name)
        # CHECK: builtin.array
        print(ArrayAttr.attr_name)
        # CHECK: builtin.integer
        print(IntegerAttr.attr_name)
        # CHECK: builtin.symbol_ref
        print(SymbolRefAttr.attr_name)
        # CHECK: builtin.opaque
        print(OpaqueAttr.attr_name)
        # CHECK: builtin.dictionary
        print(DictAttr.attr_name)
        # CHECK: builtin.type
        print(TypeAttr.attr_name)
        # CHECK: builtin.unit
        print(UnitAttr.attr_name)
        # CHECK: builtin.strided_layout
        print(StridedLayoutAttr.attr_name)
        # CHECK: builtin.dense_resource_elements
        print(DenseResourceElementsAttr.attr_name)
        # CHECK: builtin.string
        print(StringAttr.attr_name)
        # CHECK: builtin.float
        print(FloatAttr.attr_name)


# CHECK-LABEL: TEST: testLargeIntegerAttr
@run
def testLargeIntegerAttr():
    with Context():
        # Test 128-bit unsigned integer
        i128_type = IntegerType.get_unsigned(128)
        large_value_128 = (1 << 127) + 12345
        attr_128 = IntegerAttr.get(i128_type, large_value_128)
        # CHECK: 128-bit value matches: True
        print("128-bit value matches:", int(attr_128) == large_value_128)

        # Test 256-bit unsigned integer (BN254 field modulus example)
        i256_type = IntegerType.get_unsigned(256)
        bn254_modulus = 21888242871839275222246405745257275088548364400416034343698204186575808495617
        attr_256 = IntegerAttr.get(i256_type, bn254_modulus)
        # CHECK: 256-bit value matches: True
        print("256-bit value matches:", int(attr_256) == bn254_modulus)

        # Test 256-bit signed integer (positive value)
        si256_type = IntegerType.get_signed(256)
        positive_signed = (1 << 200) + 999
        attr_si256_pos = IntegerAttr.get(si256_type, positive_signed)
        # CHECK: 256-bit signed positive matches: True
        print(
            "256-bit signed positive matches:", int(attr_si256_pos) == positive_signed
        )

        # Test 256-bit signed integer (negative value)
        negative_signed = -((1 << 200) + 12345)
        attr_si256_neg = IntegerAttr.get(si256_type, negative_signed)
        # CHECK: 256-bit signed negative matches: True
        print(
            "256-bit signed negative matches:", int(attr_si256_neg) == negative_signed
        )

        # Test 64-bit boundary (should still work with fast path)
        i64_type = IntegerType.get_signless(64)
        value_64 = (1 << 63) - 1  # max signed 64-bit
        attr_64 = IntegerAttr.get(i64_type, value_64)
        # CHECK: 64-bit value matches: True
        print("64-bit value matches:", int(attr_64) == value_64)

        # Test edge case: 65-bit integer (just over 64-bit boundary)
        i65_type = IntegerType.get_unsigned(65)
        value_65 = (1 << 64) + 1
        attr_65 = IntegerAttr.get(i65_type, value_65)
        # CHECK: 65-bit value matches: True
        print("65-bit value matches:", int(attr_65) == value_65)

        # Test very large integer (512-bit)
        i512_type = IntegerType.get_unsigned(512)
        value_512 = (1 << 500) + (1 << 300) + (1 << 100) + 42
        attr_512 = IntegerAttr.get(i512_type, value_512)
        # CHECK: 512-bit value matches: True
        print("512-bit value matches:", int(attr_512) == value_512)



"""
Test lldb data formatter subsystem.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class GenericListDataFormatterTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False
    TEST_WITH_PDB_DEBUG_INFO = True

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line numbers to break at for the different tests.
        self.line = line_number("main.cpp", "// Set break point at this line.")
        self.optional_line = line_number(
            "main.cpp", "// Optional break point at this line."
        )
        self.final_line = line_number(
            "main.cpp", "// Set final break point at this line."
        )

    def do_test_with_run_command(self, *, is_libstdcpp=False):
        """Test that that file and class static variables display correctly."""
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=-1
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type format clear", check=False)
            self.runCmd("type summary clear", check=False)
            self.runCmd("type filter clear", check=False)
            self.runCmd("type synth clear", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        self.runCmd("frame variable numbers_list --show-types")

        self.runCmd("type format add -f hex int")

        self.expect(
            "frame variable numbers_list --raw", matching=False, substrs=["size=0"]
        )

        if is_libstdcpp:
            self.expect(
                "frame variable &numbers_list._M_impl._M_node --raw",
                matching=False,
                substrs=["size=0", "{}"],
            )

        self.expect("frame variable numbers_list", substrs=["size=0", "{}"])

        self.expect("expression numbers_list", substrs=["size=0", "{}"])

        self.runCmd("n")

        self.expect(
            "frame variable numbers_list", substrs=["size=1", "[0] = ", "0x12345678"]
        )

        self.runCmd("n")
        self.runCmd("n")
        self.runCmd("n")

        self.expect(
            "frame variable numbers_list",
            substrs=[
                "size=4",
                "[0] = ",
                "0x12345678",
                "[1] =",
                "0x11223344",
                "[2] =",
                "0xbeeffeed",
                "[3] =",
                "0x00abba00",
            ],
        )

        self.runCmd("n")
        self.runCmd("n")

        self.expect(
            "frame variable numbers_list",
            substrs=[
                "size=6",
                "[0] = ",
                "0x12345678",
                "0x11223344",
                "0xbeeffeed",
                "0x00abba00",
                "[4] =",
                "0x0abcdef0",
                "[5] =",
                "0x0cab0cab",
            ],
        )

        self.expect(
            "expression numbers_list",
            substrs=[
                "size=6",
                "[0] = ",
                "0x12345678",
                "0x11223344",
                "0xbeeffeed",
                "0x00abba00",
                "[4] =",
                "0x0abcdef0",
                "[5] =",
                "0x0cab0cab",
            ],
        )

        # check access-by-index
        self.expect("frame variable numbers_list[0]", substrs=["0x12345678"])
        self.expect("frame variable numbers_list[1]", substrs=["0x11223344"])

        # but check that expression does not rely on us
        self.expect(
            "expression numbers_list[0]",
            matching=False,
            error=True,
            substrs=["0x12345678"],
        )

        # check that MightHaveChildren() gets it right
        self.assertTrue(
            self.frame().FindVariable("numbers_list").MightHaveChildren(),
            "numbers_list.MightHaveChildren() says False for non empty!",
        )

        self.runCmd("n")

        self.expect("frame variable numbers_list", substrs=["size=0", "{}"])

        self.runCmd("n")
        self.runCmd("n")
        self.runCmd("n")
        self.runCmd("n")

        self.expect(
            "frame variable numbers_list",
            substrs=[
                "size=4",
                "[0] = ",
                "1",
                "[1] = ",
                "2",
                "[2] = ",
                "3",
                "[3] = ",
                "4",
            ],
        )

        self.runCmd("type format delete int")

        lldbutil.run_break_set_by_file_and_line(self, "main.cpp", self.optional_line)
        self.runCmd("continue")

        self.expect("frame variable text_list", substrs=["size=0", "{}"])

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.final_line, num_expected_locations=-1
        )

        self.runCmd("c", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        self.expect(
            "frame variable text_list",
            substrs=[
                "size=4",
                "[0]",
                "goofy",
                "[1]",
                "is",
                "[2]",
                "smart",
                "[3]",
                "!!!",
            ],
        )

        self.expect(
            "expression text_list",
            substrs=["size=4", '"goofy"', '"is"', '"smart"', '"!!!"'],
        )

        # check access-by-index
        self.expect("frame variable text_list[0]", substrs=["goofy"])
        self.expect("frame variable text_list[3]", substrs=["!!!"])

        # but check that expression does not rely on us
        self.expect(
            "expression text_list[0]", matching=False, error=True, substrs=["goofy"]
        )

        # check that MightHaveChildren() gets it right
        self.assertTrue(
            self.frame().FindVariable("text_list").MightHaveChildren(),
            "text_list.MightHaveChildren() says False for non empty!",
        )

    def do_test_ptr_and_ref(self):
        """Test that ref and ptr to std::list is displayed correctly"""
        (_, process, _, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Check ref and ptr", lldb.SBFileSpec("main.cpp", False)
        )

        self.expect(
            "frame variable ref",
            substrs=[
                "size=4",
                "[0] = ",
                "1",
                "[1] = ",
                "2",
                "[2] = ",
                "3",
                "[3] = ",
                "4",
            ],
        )

        self.expect(
            "frame variable *ptr",
            substrs=[
                "size=4",
                "[0] = ",
                "1",
                "[1] = ",
                "2",
                "[2] = ",
                "3",
                "[3] = ",
                "4",
            ],
        )

        lldbutil.continue_to_breakpoint(process, bkpt)

        self.expect(
            "frame variable ref",
            substrs=[
                "size=4",
                "[0]",
                "goofy",
                "[1]",
                "is",
                "[2]",
                "smart",
                "[3]",
                "!!!",
            ],
        )

        self.expect(
            "frame variable *ptr",
            substrs=[
                "size=4",
                "[0]",
                "goofy",
                "[1]",
                "is",
                "[2]",
                "smart",
                "[3]",
                "!!!",
            ],
        )

    @add_test_categories(["libstdcxx"])
    def test_with_run_command_libstdcpp(self):
        self.build(dictionary={"USE_LIBSTDCPP": 1})
        self.do_test_with_run_command(is_libstdcpp=True)

    @add_test_categories(["libstdcxx"])
    def test_ptr_and_ref_libstdcpp(self):
        self.build(dictionary={"USE_LIBSTDCPP": 1})
        self.do_test_ptr_and_ref()

    @add_test_categories(["libc++"])
    def test_with_run_command_libcpp(self):
        self.build(dictionary={"USE_LIBCPP": 1})
        self.do_test_with_run_command()

    @add_test_categories(["libc++"])
    def test_ptr_and_ref_libcpp(self):
        self.build(dictionary={"USE_LIBCPP": 1})
        self.do_test_ptr_and_ref()

    @add_test_categories(["msvcstl"])
    def test_with_run_command_msvcstl(self):
        # No flags, because the "msvcstl" category checks that the MSVC STL is used by default.
        self.build()
        self.do_test_with_run_command()

    @add_test_categories(["msvcstl"])
    def test_ptr_and_ref_msvcstl(self):
        self.build()
        self.do_test_ptr_and_ref()



"""
Test some lldb command abbreviations.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ExecTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @expectedFailureAll(
        archs=["i386"], oslist=no_match(["freebsd"]), bugnumber="rdar://28656532"
    )
    @expectedFailureAll(
        oslist=["ios", "tvos", "watchos", "bridgeos"],
        bugnumber="rdar://problem/34559552",
    )  # this exec test has problems on ios systems
    @expectedFailureNetBSD
    @skipIfAsan  # rdar://problem/43756823
    @skipIfWindows
    def test_hitting_exec(self):
        self.do_test(False)

    @expectedFailureAll(
        archs=["i386"], oslist=no_match(["freebsd"]), bugnumber="rdar://28656532"
    )
    @expectedFailureAll(
        oslist=["ios", "tvos", "watchos", "bridgeos"],
        bugnumber="rdar://problem/34559552",
    )  # this exec test has problems on ios systems
    @expectedFailureNetBSD
    @skipIfAsan  # rdar://problem/43756823
    @skipIfWindows
    def test_skipping_exec(self):
        self.do_test(True)

    def do_test(self, skip_exec):
        self.build()
        exe = self.getBuildArtifact("a.out")
        secondprog = self.getBuildArtifact("secondprog")

        # Create the target
        target = self.dbg.CreateTarget(exe)

        lldbutil.install_to_target(self, secondprog)

        # Create any breakpoints we need
        breakpoint1 = target.BreakpointCreateBySourceRegex(
            "Set breakpoint 1 here", lldb.SBFileSpec("main.c", False)
        )
        self.assertTrue(breakpoint1, VALID_BREAKPOINT)
        breakpoint2 = target.BreakpointCreateBySourceRegex(
            "Set breakpoint 2 here", lldb.SBFileSpec("secondprog.cpp", False)
        )
        self.assertTrue(breakpoint2, VALID_BREAKPOINT)

        # Launch the process
        process = target.LaunchSimple(None, None, self.get_process_working_directory())
        self.assertTrue(process, PROCESS_IS_VALID)

        if self.TraceOn():
            self.runCmd("settings show target.process.stop-on-exec", check=False)
        if skip_exec:
            self.dbg.HandleCommand("settings set target.process.stop-on-exec false")

            def cleanup():
                self.runCmd(
                    "settings set target.process.stop-on-exec false", check=False
                )

            # Execute the cleanup function during test case tear down.
            self.addTearDownHook(cleanup)

        # The stop reason of the thread should be breakpoint.
        self.assertState(
            process.GetState(), lldb.eStateStopped, STOPPED_DUE_TO_BREAKPOINT
        )

        threads = lldbutil.get_threads_stopped_at_breakpoint(process, breakpoint1)
        self.assertEqual(len(threads), 1)

        # We had a deadlock tearing down the TypeSystemMap on exec, but only if some
        # expression had been evaluated.  So make sure we do that here so the teardown
        # is not trivial.

        thread = threads[0]
        value = thread.frames[0].EvaluateExpression("1 + 2")
        self.assertTrue(value.IsValid(), "Expression evaluated successfully")
        int_value = value.GetValueAsSigned()
        self.assertEqual(int_value, 3, "Expression got the right result.")

        # Run and we should stop due to exec
        process.Continue()

        if not skip_exec:
            self.assertNotEqual(
                process.GetState(), lldb.eStateExited, "Process should not have exited!"
            )
            self.assertState(
                process.GetState(),
                lldb.eStateStopped,
                "Process should be stopped at __dyld_start",
            )

            threads = lldbutil.get_stopped_threads(process, lldb.eStopReasonExec)
            self.assertEqual(len(threads), 1, "We got a thread stopped for exec.")

            # Run and we should stop at breakpoint in main after exec
            process.Continue()

        self.assertState(process.GetState(), lldb.eStateStopped)
        for t in process.threads:
            if t.stop_reason != lldb.eStopReasonNone:
                self.assertStopReason(
                    t.stop_reason, lldb.eStopReasonBreakpoint, "Unexpected stop reason"
                )
                if self.TraceOn():
                    print(t)
                    if t.stop_reason != lldb.eStopReasonBreakpoint:
                        self.runCmd("bt")

        threads = lldbutil.get_threads_stopped_at_breakpoint(process, breakpoint2)
        self.assertEqual(len(threads), 1, "Stopped at breakpoint in exec'ed process.")

    @expectedFailureAll(
        archs=["i386"], oslist=no_match(["freebsd"]), bugnumber="rdar://28656532"
    )
    @expectedFailureAll(
        oslist=["ios", "tvos", "watchos", "bridgeos"],
        bugnumber="rdar://problem/34559552",
    )  # this exec test has problems on ios systems
    @expectedFailureNetBSD
    @skipIfAsan  # rdar://problem/43756823
    @skipIfWindows
    def test_correct_thread_plan_state_before_exec(self):
        """
        In this test we make sure that the Thread* cache in the ThreadPlans
        is cleared correctly when performing exec
        """

        self.build()
        exe = self.getBuildArtifact("a.out")
        target = self.dbg.CreateTarget(exe)

        lldbutil.install_to_target(self, self.getBuildArtifact("secondprog"))

        (target, process, thread, breakpoint1) = lldbutil.run_to_source_breakpoint(
            self, "Set breakpoint 1 here", lldb.SBFileSpec("main.c", False)
        )

        # The stop reason of the thread should be breakpoint.
        self.assertState(
            process.GetState(), lldb.eStateStopped, STOPPED_DUE_TO_BREAKPOINT
        )

        threads = lldbutil.get_threads_stopped_at_breakpoint(process, breakpoint1)
        self.assertEqual(len(threads), 1)

        # We perform an instruction step, which effectively sets the cache of the base
        # thread plan, which should be cleared when a new thread list appears.
        #
        # Continuing after this instruction step will trigger a call to
        # ThreadPlan::ShouldReportRun, which sets the ThreadPlan's Thread cache to
        # the old Thread* value. In Process::UpdateThreadList we are clearing this
        # cache in preparation for the new ThreadList.
        #
        # Not doing this stepping will cause LLDB to first execute a private single step
        # past the current breakpoint, which eventually avoids the call to ShouldReportRun,
        # thus not setting the cache to its invalid value.
        thread.StepInstruction(False)

        # Run and we should stop due to exec
        breakpoint2 = target.BreakpointCreateBySourceRegex(
            "Set breakpoint 2 here", lldb.SBFileSpec("secondprog.cpp", False)
        )

        process.Continue()

        self.assertNotEqual(
            process.GetState(), lldb.eStateExited, "Process should not have exited!"
        )
        self.assertState(
            process.GetState(),
            lldb.eStateStopped,
            "Process should be stopped at __dyld_start",
        )

        threads = lldbutil.get_stopped_threads(process, lldb.eStopReasonExec)
        self.assertEqual(len(threads), 1, "We got a thread stopped for exec.")

        # Run and we should stop at breakpoint in main after exec
        process.Continue()

        threads = lldbutil.get_threads_stopped_at_breakpoint(process, breakpoint2)
        self.assertEqual(len(threads), 1, "Stopped at breakpoint in exec'ed process.")



"""
Test that the user can input a format but it will not prevail over summary format's choices.
"""


import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class UserFormatVSSummaryTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break at.
        self.line = line_number("main.cpp", "// Set break point at this line.")

    def test_with_run_command(self):
        """Test that the user can input a format but it will not prevail over summary format's choices."""
        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        self.expect("frame variable p1", substrs=["(Pair) p1 = (x = 3, y = -3)"])

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type format clear", check=False)
            self.runCmd("type summary clear", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        self.runCmd('type summary add Pair -s "x=${var.x%d},y=${var.y%u}"')

        self.expect("frame variable p1", substrs=["(Pair) p1 = x=3,y=4294967293"])
        self.expect(
            "frame variable -f x p1",
            substrs=["(Pair) p1 = x=0x00000003,y=0xfffffffd"],
            matching=False,
        )
        self.expect(
            "frame variable -f d p1", substrs=["(Pair) p1 = x=3,y=-3"], matching=False
        )
        self.expect("frame variable p1", substrs=["(Pair) p1 = x=3,y=4294967293"])

        self.runCmd('type summary add Pair -s "x=${var.x%x},y=${var.y%u}"')

        self.expect(
            "frame variable p1", substrs=["(Pair) p1 = x=0x00000003,y=4294967293"]
        )
        self.expect(
            "frame variable -f d p1", substrs=["(Pair) p1 = x=3,y=-3"], matching=False
        )



from lldbsuite.test import lldbinline, lldbplatformutil
from lldbsuite.test import decorators

lldbinline.MakeInlineTest(
    __file__,
    globals(),
    [decorators.expectedFailureAll(bugnumber="llvm.org/PR36715", oslist=["windows"])],
)



"""
Test lldb data formatter subsystem.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class PrintObjectArrayTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False

    @skipUnlessDarwin
    def test_print_array(self):
        """Test that expr -O -Z works"""
        self.build()
        self.printarray_data_formatter_commands()

    @skipUnlessDarwin
    def test_print_array_no_const(self):
        """Test that expr -O -Z works"""
        disable_constant_classes = {
            "USE_SYSTEM_STDLIB": "1",  # See above.
            "CFLAGS_EXTRAS": "-fno-constant-nsnumber-literals "
            + "-fno-constant-nsarray-literals "
            + "-fno-constant-nsdictionary-literals",
        }
        # FIXME: Remove compiler when flags are available upstream.
        self.build(dictionary=disable_constant_classes, compiler="xcrun clang")
        self.printarray_data_formatter_commands()

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break at.
        self.line = line_number("main.mm", "break here")

    def printarray_data_formatter_commands(self):
        """Test that expr -O -Z works"""
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.mm", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type format clear", check=False)
            self.runCmd("type summary clear", check=False)
            self.runCmd("type synth clear", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        self.expect(
            "expr --element-count 3 --object-description -- objects",
            substrs=[
                "3735928559",
                "4276993775",
                "3203398366",
                "Hello",
                "World",
                "Two =",
                "1 =",
            ],
        )
        self.expect(
            "poarray 3 objects",
            substrs=[
                "3735928559",
                "4276993775",
                "3203398366",
                "Hello",
                "World",
                "Two =",
                "1 =",
            ],
        )
        self.expect(
            "expr --element-count 3 --object-description --description-verbosity=full -- objects",
            substrs=[
                "[0] =",
                "3735928559",
                "4276993775",
                "3203398366",
                "[1] =",
                "Hello",
                "World",
                "[2] =",
                "Two =",
                "1 =",
            ],
        )
        self.expect("parray 3 objects", substrs=["[0] = 0x", "[1] = 0x", "[2] = 0x"])
        self.expect(
            "expr --element-count 3 -d run -- objects",
            substrs=["3 elements", "2 elements", "2 key/value pairs"],
        )
        self.expect(
            "expr --element-count 3 -d run --ptr-depth=1 -- objects",
            substrs=[
                "3 elements",
                "3735928559",
                "4276993775",
                "3203398366",
                "2 elements",
                '"Hello"',
                '"World"',
                "2 key/value pairs",
            ],
        )



import gdbremote_testcase


class GdbRemoteForkTestBase(gdbremote_testcase.GdbRemoteTestCaseBase):
    fork_regex = (
        "[$]T[0-9a-fA-F]{{2}}thread:p([0-9a-f]+)[.]([0-9a-f]+);.*"
        "{}:p([0-9a-f]+)[.]([0-9a-f]+).*"
    )
    fork_regex_nonstop = (
        "%Stop:T[0-9a-fA-F]{{2}}"
        "thread:p([0-9a-f]+)[.]([0-9a-f]+);.*"
        "{}:p([0-9a-f]+)[.]([0-9a-f]+).*"
    )
    fork_capture = {1: "parent_pid", 2: "parent_tid", 3: "child_pid", 4: "child_tid"}
    stop_regex_base = "T[0-9a-fA-F]{{2}}thread:p{}.{};.*reason:signal.*"
    stop_regex = "^[$]" + stop_regex_base

    def start_fork_test(self, args, variant="fork", nonstop=False):
        self.build()
        self.prep_debug_monitor_and_inferior(inferior_args=args)
        self.add_qSupported_packets(["multiprocess+", "{}-events+".format(variant)])
        ret = self.expect_gdbremote_sequence()
        self.assertIn("{}-events+".format(variant), ret["qSupported_response"])
        self.reset_test_sequence()

        # continue and expect fork
        if nonstop:
            self.test_sequence.add_log_lines(
                [
                    "read packet: $QNonStop:1#00",
                    "send packet: $OK#00",
                    "read packet: $c#00",
                    "send packet: $OK#00",
                    {
                        "direction": "send",
                        "regex": self.fork_regex_nonstop.format(variant),
                        "capture": self.fork_capture,
                    },
                    "read packet: $vStopped#00",
                    "send packet: $OK#00",
                ],
                True,
            )
        else:
            self.test_sequence.add_log_lines(
                [
                    "read packet: $c#00",
                    {
                        "direction": "send",
                        "regex": self.fork_regex.format(variant),
                        "capture": self.fork_capture,
                    },
                ],
                True,
            )
        ret = self.expect_gdbremote_sequence()
        self.reset_test_sequence()

        return tuple(
            ret[x] for x in ("parent_pid", "parent_tid", "child_pid", "child_tid")
        )

    def fork_and_detach_test(self, variant, nonstop=False):
        parent_pid, parent_tid, child_pid, child_tid = self.start_fork_test(
            [variant], variant, nonstop=nonstop
        )

        # detach the forked child
        self.test_sequence.add_log_lines(
            [
                "read packet: $D;{}#00".format(child_pid),
                "send packet: $OK#00",
                # verify that the current process is correct
                "read packet: $qC#00",
                "send packet: $QCp{}.{}#00".format(parent_pid, parent_tid),
                # verify that the correct processes are detached/available
                "read packet: $Hgp{}.{}#00".format(child_pid, child_tid),
                "send packet: $Eff#00",
                "read packet: $Hgp{}.{}#00".format(parent_pid, parent_tid),
                "send packet: $OK#00",
            ],
            True,
        )
        self.expect_gdbremote_sequence()
        self.reset_test_sequence()
        return parent_pid, parent_tid

    def fork_and_follow_test(self, variant, nonstop=False):
        parent_pid, parent_tid, child_pid, child_tid = self.start_fork_test(
            [variant], variant, nonstop=nonstop
        )

        # switch to the forked child
        self.test_sequence.add_log_lines(
            [
                "read packet: $Hgp{}.{}#00".format(child_pid, child_tid),
                "send packet: $OK#00",
                "read packet: $Hcp{}.{}#00".format(child_pid, child_tid),
                "send packet: $OK#00",
                # detach the parent
                "read packet: $D;{}#00".format(parent_pid),
                "send packet: $OK#00",
                # verify that the correct processes are detached/available
                "read packet: $Hgp{}.{}#00".format(parent_pid, parent_tid),
                "send packet: $Eff#00",
                "read packet: $Hgp{}.{}#00".format(child_pid, child_tid),
                "send packet: $OK#00",
                # then resume the child
                "read packet: $c#00",
            ],
            True,
        )

        if nonstop:
            self.test_sequence.add_log_lines(
                [
                    "send packet: $OK#00",
                    "send packet: %Stop:W00;process:{}#00".format(child_pid),
                    "read packet: $vStopped#00",
                    "send packet: $OK#00",
                ],
                True,
            )
        else:
            self.test_sequence.add_log_lines(
                [
                    "send packet: $W00;process:{}#00".format(child_pid),
                ],
                True,
            )
        self.expect_gdbremote_sequence()

    def detach_all_test(self, nonstop=False):
        parent_pid, parent_tid, child_pid, child_tid = self.start_fork_test(
            ["fork"], nonstop=nonstop
        )

        self.test_sequence.add_log_lines(
            [
                # double-check our PIDs
                "read packet: $Hgp{}.{}#00".format(parent_pid, parent_tid),
                "send packet: $OK#00",
                "read packet: $Hgp{}.{}#00".format(child_pid, child_tid),
                "send packet: $OK#00",
                # detach all processes
                "read packet: $D#00",
                "send packet: $OK#00",
                # verify that both PIDs are invalid now
                "read packet: $Hgp{}.{}#00".format(parent_pid, parent_tid),
                "send packet: $Eff#00",
                "read packet: $Hgp{}.{}#00".format(child_pid, child_tid),
                "send packet: $Eff#00",
            ],
            True,
        )
        self.expect_gdbremote_sequence()

    def vkill_test(self, kill_parent=False, kill_child=False, nonstop=False):
        assert kill_parent or kill_child
        parent_pid, parent_tid, child_pid, child_tid = self.start_fork_test(
            ["fork"], nonstop=nonstop
        )

        if kill_parent:
            self.test_sequence.add_log_lines(
                [
                    # kill the process
                    "read packet: $vKill;{}#00".format(parent_pid),
                    "send packet: $OK#00",
                ],
                True,
            )
        if kill_child:
            self.test_sequence.add_log_lines(
                [
                    # kill the process
                    "read packet: $vKill;{}#00".format(child_pid),
                    "send packet: $OK#00",
                ],
                True,
            )
        self.test_sequence.add_log_lines(
            [
                # check child PID/TID
                "read packet: $Hgp{}.{}#00".format(child_pid, child_tid),
                "send packet: ${}#00".format("Eff" if kill_child else "OK"),
                # check parent PID/TID
                "read packet: $Hgp{}.{}#00".format(parent_pid, parent_tid),
                "send packet: ${}#00".format("Eff" if kill_parent else "OK"),
            ],
            True,
        )
        self.expect_gdbremote_sequence()

    def resume_one_test(self, run_order, use_vCont=False, nonstop=False):
        parent_pid, parent_tid, child_pid, child_tid = self.start_fork_test(
            ["fork", "stop"], nonstop=nonstop
        )

        parent_expect = [
            self.stop_regex_base.format(parent_pid, parent_tid),
            "W00;process:{}#.*".format(parent_pid),
        ]
        child_expect = [
            self.stop_regex_base.format(child_pid, child_tid),
            "W00;process:{}#.*".format(child_pid),
        ]

        for x in run_order:
            if x == "parent":
                pidtid = (parent_pid, parent_tid)
                expect = parent_expect.pop(0)
            elif x == "child":
                pidtid = (child_pid, child_tid)
                expect = child_expect.pop(0)
            else:
                assert False, "unexpected x={}".format(x)

            if use_vCont:
                self.test_sequence.add_log_lines(
                    [
                        # continue the selected process
                        "read packet: $vCont;c:p{}.{}#00".format(*pidtid),
                    ],
                    True,
                )
            else:
                self.test_sequence.add_log_lines(
                    [
                        # continue the selected process
                        "read packet: $Hcp{}.{}#00".format(*pidtid),
                        "send packet: $OK#00",
                        "read packet: $c#00",
                    ],
                    True,
                )
            if nonstop:
                self.test_sequence.add_log_lines(
                    [
                        "send packet: $OK#00",
                        {"direction": "send", "regex": "%Stop:" + expect},
                        "read packet: $vStopped#00",
                        "send packet: $OK#00",
                    ],
                    True,
                )
            else:
                self.test_sequence.add_log_lines(
                    [
                        {"direction": "send", "regex": "[$]" + expect},
                    ],
                    True,
                )
            # if at least one process remained, check both PIDs
            if parent_expect or child_expect:
                self.test_sequence.add_log_lines(
                    [
                        "read packet: $Hgp{}.{}#00".format(parent_pid, parent_tid),
                        "send packet: ${}#00".format("OK" if parent_expect else "Eff"),
                        "read packet: $Hgp{}.{}#00".format(child_pid, child_tid),
                        "send packet: ${}#00".format("OK" if child_expect else "Eff"),
                    ],
                    True,
                )
        self.expect_gdbremote_sequence()



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class StaticInitializers(TestBase):
    @expectedFailureAll(
        archs="aarch64",
        oslist=["freebsd"],
        bugnumber="llvm.org/pr44053",
    )
    def test(self):
        """Test a static initializer."""
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp", False)
        )

        # We use counter to observe if the initializer was called.
        self.expect_expr("counter", result_type="int", result_value="0")
        self.expect("expr -p -- struct Foo { Foo() { inc_counter(); } }; Foo f;")
        self.expect_expr("counter", result_type="int", result_value="1")

    def test_failing_init(self):
        """Test a static initializer that fails to execute."""
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp", False)
        )

        # FIXME: This error message is not even remotely helpful.
        self.expect(
            "expr -p -- struct Foo2 { Foo2() { do_abort(); } }; Foo2 f;",
            error=True,
            substrs=["couldn't run static initializer:"],
        )

    def test_without_process(self):
        """Test a static initializer without a running process."""
        self.expect(
            "expr -p -- int i = 0; struct Foo3 { Foo3() { ++i; } }; Foo3 f;",
            error=True,
            substrs=["Top-level code needs to be inserted into a runnable target"],
        )



"""
Make sure 'frame var' using DIL parser/evaultor works for C-Style casts.
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test import lldbutil

import os
import shutil
import time


class TestFrameVarDILCast(TestBase):
    def test_type_cast(self):
        self.build()
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", lldb.SBFileSpec("main.cpp")
        )

        self.runCmd("settings set target.experimental.use-DIL true")

        Is32Bit = False
        if self.target().GetAddressByteSize() == 4:
            Is32Bit = True

        # TestCastBUiltins

        self.expect_var_path("(int)1", value="1", type="int")
        self.expect_var_path("(long long)1", value="1", type="long long")
        self.expect_var_path("(unsigned long)1", value="1", type="unsigned long")
        if Is32Bit:
            self.expect_var_path("(char*)1", value="0x00000001", type="char *")
            self.expect_var_path(
                "(long long**)1", value="0x00000001", type="long long **"
            )
        else:
            self.expect_var_path("(char*)1", value="0x0000000000000001", type="char *")
            self.expect_var_path(
                "(long long**)1", value="0x0000000000000001", type="long long **"
            )

        self.expect(
            "frame variable '(long&*)1'",
            error=True,
            substrs=[
                "'type name' declared as a pointer to a reference of type 'long &'"
            ],
        )

        self.expect(
            "frame variable '(long& &)1'",
            error=True,
            substrs=["type name declared as a reference to a reference"],
        )

        self.expect(
            "frame variable '(long 1)1'",
            error=True,
            substrs=["expected 'r_paren', got: <'1' (integer_constant)>"],
        )

        # TestCastBasicType

        # Test with integer literals.
        self.expect_var_path("(char)1", type="char", value="'\\x01'")
        self.expect_var_path("(long long)1", type="long long", value="1")
        self.expect_var_path("(short)65534", type="short", value="-2")
        self.expect_var_path(
            "(unsigned short)100000", type="unsigned short", value="34464"
        )
        self.expect_var_path("(int)false", type="int", value="0")
        self.expect_var_path("(int)true", type="int", value="1")
        self.expect_var_path("(float)1", type="float", value="1")
        self.expect_var_path("(float)1.1", type="float", value="1.10000002")
        self.expect_var_path("(float)1.1f", type="float", value="1.10000002")
        self.expect_var_path("(float)false", type="float", value="0")
        self.expect_var_path("(float)true", type="float", value="1")
        self.expect_var_path("(double)1", type="double", value="1")
        self.expect_var_path("(double)1.1", type="double", value="1.1000000000000001")
        self.expect_var_path("(double)1.1f", type="double", value="1.1000000238418579")
        self.expect_var_path("(double)false", type="double", value="0")
        self.expect_var_path("(double)true", type="double", value="1")
        self.expect_var_path("(int)1.1", type="int", value="1")
        self.expect_var_path("(int)1.1f", type="int", value="1")
        self.expect_var_path("(long)1.1", type="long", value="1")
        self.expect_var_path("(bool)0", type="bool", value="false")
        self.expect_var_path("(bool)0.0", type="bool", value="false")
        self.expect_var_path("(bool)0.0f", type="bool", value="false")
        self.expect_var_path("(bool)3", type="bool", value="true")

        self.expect(
            "frame variable '&(int)1'",
            error=True,
            substrs=["'result' doesn't have a valid address"],
        )

        # Test with variables.
        self.expect_var_path("(char)a", type="char", value="'\\x01'")
        self.expect_var_path("(unsigned char)na", type="unsigned char", value="'\\xff'")
        self.expect_var_path("(short)na", type="short", value="-1")
        self.expect_var_path("(long long)a", type="long long", value="1")
        self.expect_var_path("(float)a", type="float", value="1")
        self.expect_var_path("(float)f", type="float", value="1.10000002")
        self.expect_var_path("(double)f", type="double", value="1.1000000238418579")
        self.expect_var_path("(int)f", type="int", value="1")
        self.expect_var_path("(long)f", type="long", value="1")
        self.expect_var_path("(bool)finf", type="bool", value="true")
        self.expect_var_path("(bool)fnan", type="bool", value="true")
        self.expect_var_path("(bool)fsnan", type="bool", value="true")
        self.expect_var_path("(bool)fmax", type="bool", value="true")
        self.expect_var_path("(bool)fdenorm", type="bool", value="true")
        self.expect(
            "frame variable '(int)ns_foo_'",
            error=True,
            substrs=["cannot convert 'ns::Foo' to 'int'"],
        )

        # Test with typedefs and namespaces.
        self.expect_var_path("(myint)1", type="myint", value="1")
        self.expect_var_path("(myint)1LL", type="myint", value="1")
        self.expect_var_path("(ns::myint)1", type="ns::myint", value="1")
        self.expect_var_path("(::ns::myint)1", type="ns::myint", value="1")
        self.expect_var_path("(::ns::myint)myint_", type="ns::myint", value="1")

        self.expect_var_path("(int)myint_", type="int", value="1")
        self.expect_var_path("(int)ns_myint_", type="int", value="2")
        self.expect_var_path("(long long)myint_", type="long long", value="1")
        self.expect_var_path("(long long)ns_myint_", type="long long", value="2")
        self.expect_var_path("(::ns::myint)myint_", type="ns::myint", value="1")

        self.expect_var_path(
            "(ns::inner::mydouble)1", type="ns::inner::mydouble", value="1"
        )
        self.expect_var_path(
            "(::ns::inner::mydouble)1.2", type="ns::inner::mydouble", value="1.2"
        )
        self.expect_var_path(
            "(ns::inner::mydouble)myint_", type="ns::inner::mydouble", value="1"
        )
        self.expect_var_path(
            "(::ns::inner::mydouble)ns_inner_mydouble_",
            type="ns::inner::mydouble",
            value="1.2",
        )
        self.expect_var_path("(myint)ns_inner_mydouble_", type="myint", value="1")

        # Test with pointers and arrays.
        self.expect_var_path("(long long)ap", type="long long")
        self.expect_var_path("(unsigned long long)vp", type="unsigned long long")
        self.expect_var_path("(long long)arr", type="long long")
        self.expect_var_path("(bool)ap", type="bool", value="true")
        self.expect_var_path("(bool)(int*)0x00000000", type="bool", value="false")
        self.expect_var_path("(bool)std_nullptr_t", value="false")
        self.expect_var_path("(bool)arr", type="bool", value="true")
        self.expect(
            "frame variable '(char)ap'",
            error=True,
            substrs=["cast from pointer to smaller type 'char' loses information"],
        )

        if Is32Bit:
            self.expect_var_path("(int)arr", type="int")
        else:
            self.expect(
                "frame variable '(int)arr'",
                error=True,
                substrs=["cast from pointer to smaller type 'int' loses information"],
            )

        self.expect(
            "frame variable '(float)ap'",
            error=True,
            substrs=["Cast from 'int *' to 'float' is not allowed"],
        )
        self.expect(
            "frame variable '(float)arr'",
            error=True,
            substrs=["Cast from 'int *' to 'float' is not allowed"],
        )

        # TestCastPointer
        self.expect_var_path("(void*)&a", type="void *")
        self.expect_var_path("(void*)ap", type="void *")
        self.expect_var_path("(long long*)vp", type="long long *")
        self.expect_var_path("(short int*)vp", type="short *")
        self.expect_var_path("(unsigned long long*)vp", type="unsigned long long *")
        self.expect_var_path("(unsigned short int*)vp", type="unsigned short *")

        if Is32Bit:
            self.expect_var_path("(void*)0", type="void *", value="0x00000000")
            self.expect_var_path("(void*)1", type="void *", value="0x00000001")
            self.expect_var_path("(void*)a", type="void *", value="0x00000001")
            self.expect_var_path("(void*)na", type="void *", value="0xffffffff")
        else:
            self.expect_var_path("(void*)0", type="void *", value="0x0000000000000000")
            self.expect_var_path("(void*)1", type="void *", value="0x0000000000000001")
            self.expect_var_path("(void*)a", type="void *", value="0x0000000000000001")
            self.expect_var_path("(void*)na", type="void *", value="0xffffffffffffffff")

        self.expect(
            "frame variable '(char*) 1.0'",
            error=True,
            substrs=["cannot cast from type 'double' to pointer type 'char *'"],
        )
        self.expect_var_path("(ns::Foo*)ns_inner_foo_ptr_", type="ns::Foo *")
        self.expect_var_path("(ns::inner::Foo*)ns_foo_ptr_", type="ns::inner::Foo *")

        self.expect_var_path("*(int*)(void*)ap", type="int", value="1")

        self.expect(
            "frame variable '(int& &)ap'",
            error=True,
            substrs=["type name declared as a reference to a reference"],
        )
        self.expect(
            "frame variable '(int&*)ap'",
            error=True,
            substrs=[
                "'type name' declared as a pointer to a reference of type 'int &'"
            ],
        )

        if Is32Bit:
            self.expect_var_path("(void *)0", type="void *", value="0x00000000")
        else:
            self.expect_var_path("(void *)0", type="void *", value="0x0000000000000000")

            self.expect(
                "frame variable '(int)std_nullptr_t'",
                error=True,
                substrs=["cast from pointer to smaller type 'int' loses information"],
            )

        if Is32Bit:
            self.expect_var_path(
                "(void*)std_nullptr_t", type="void *", value="0x00000000"
            )
            self.expect_var_path(
                "(char*)std_nullptr_t", type="char *", value="0x00000000"
            )
        else:
            self.expect_var_path(
                "(void*)std_nullptr_t", type="void *", value="0x0000000000000000"
            )
            self.expect_var_path(
                "(char*)std_nullptr_t", type="char *", value="0x0000000000000000"
            )

        # TestCastArray
        self.expect_var_path("(int*)arr_1d", type="int *")
        self.expect_var_path("(char*)arr_1d", type="char *")
        self.expect_var_path("((char*)arr_1d)[0]", type="char", value="'\\x01'")
        self.expect_var_path("((char*)arr_1d)[1]", type="char", value="'\\0'")

        # 2D arrays.
        self.expect_var_path("(int*)arr_2d", type="int *")
        self.expect_var_path("((int*)arr_2d)[1]", type="int", value="2")
        self.expect_var_path("((int*)arr_2d)[2]", type="int", value="3")
        self.expect_var_path("((int*)arr_2d[1])[1]", type="int", value="5")

        # Test casting to user-defined type with same name as variable.

        self.expect_var_path("myStruct", type="myName")
        self.expect_var_path("myName", type="int", value="37")

        self.expect_var_path("secondStruct", type="myGlobalName")
        self.expect_var_path("myGlobalName", type="bool", value="true")

        # Here 'myName' is treated as a variable, not a type, so '(myName)'
        # is parsed as a variable expression and 'InnerFoo' is unexpected,
        # and a type cast is not attempted.
        self.expect(
            "frame variable '(myName)InnerFoo'",
            error=True,
            substrs=["expected 'eof', got: <'InnerFoo' (identifier)>"],
        )

        # Here 'myGlobalName' is treated as a (global) variable, not a type,
        # so '(myGlobalName)' is parsed as a variable expression and 'InnerFoo'
        # is unexpected, and a type cast is not attempted.
        self.expect(
            "frame variable '(myGlobalName)InnerFoo'",
            error=True,
            substrs=["expected 'eof', got: <'InnerFoo' (identifier)>"],
        )

        # Check that casts are not allowed in both simple and legacy modes
        frame = thread.GetFrameAtIndex(0)
        simple = frame.GetValueForVariablePath("(char)a", lldb.eDILModeSimple)
        legacy = frame.GetValueForVariablePath("(char)a", lldb.eDILModeLegacy)
        self.assertFailure(simple.GetError())
        self.assertFailure(legacy.GetError())



from lldbsuite.test import decorators
from lldbsuite.test import lldbinline

lldbinline.MakeInlineTest(
    __file__,
    globals(),
    [
        decorators.add_test_categories(["objc"]),
        decorators.expectedFailureAll(
            oslist=["macosx"], archs=["i386"], bugnumber="rdar://28656677"
        ),
    ],
)



"""
Test lldb core component: SourceManager.

Test cases:

o test_display_source_python:
  Test display of source using the SBSourceManager API.
o test_modify_source_file_while_debugging:
  Test the caching mechanism of the source manager.
"""

import os
import io
import stat

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


def ansi_underline_surround_regex(inner_regex_text):
    # return re.compile(r"\[4m%s\[0m" % inner_regex_text)
    return "4.+\033\\[4m%s\033\\[0m" % inner_regex_text


def ansi_color_surround_regex(inner_regex_text):
    return "\033\\[3[0-7]m%s\033\\[0m" % inner_regex_text


class SourceManagerTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True
    SHARED_BUILD_TESTCASE = False

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside main().
        self.file = os.path.realpath(self.getBuildArtifact("main-copy.c"))
        self.line = line_number("main.c", "// Set break point at this line.")

    def modify_content(self):
        # Read the main.c file content.
        with io.open(self.file, "r", newline="\n") as f:
            original_content = f.read()
            if self.TraceOn():
                print("original content:", original_content)

        # Modify the in-memory copy of the original source code.
        new_content = original_content.replace("Hello world", "Hello lldb", 1)

        # Modify the source code file.
        # If the source was read only, the copy will also be read only.
        # Run "chmod u+w" on it first so we can modify it.
        statinfo = os.stat(self.file)
        os.chmod(self.file, statinfo.st_mode | stat.S_IWUSR)

        with io.open(self.file, "w", newline="\n") as f:
            time.sleep(1)
            f.write(new_content)
            if self.TraceOn():
                print("new content:", new_content)
                print(
                    "os.path.getmtime() after writing new content:",
                    os.path.getmtime(self.file),
                )

    def get_expected_stop_column_number(self):
        """Return the 1-based column number of the first non-whitespace
        character in the breakpoint source line."""
        stop_line = get_line(self.file, self.line)
        # The number of spaces that must be skipped to get to the first non-
        # whitespace character --- where we expect the debugger breakpoint
        # column to be --- is equal to the number of characters that get
        # stripped off the front when we lstrip it, plus one to specify
        # the character column after the initial whitespace.
        return len(stop_line) - len(stop_line.lstrip()) + 1

    def do_display_source_python_api(
        self, use_color, needle_regex, highlight_source=False
    ):
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Launch the process, and do not stop at the entry point.
        args = None
        envp = None
        process = target.LaunchSimple(args, envp, self.get_process_working_directory())
        self.assertIsNotNone(process)

        #
        # Exercise Python APIs to display source lines.
        #

        # Setup whether we should use ansi escape sequences, including color
        # and styles such as underline.
        self.dbg.SetUseColor(use_color)
        # Disable syntax highlighting if needed.

        self.runCmd("settings set highlight-source " + str(highlight_source).lower())

        filespec = lldb.SBFileSpec(self.file, False)
        source_mgr = self.dbg.GetSourceManager()
        # Use a string stream as the destination.
        stream = lldb.SBStream()
        column = self.get_expected_stop_column_number()
        context_before = 2
        context_after = 2
        current_line_prefix = "=>"
        source_mgr.DisplaySourceLinesWithLineNumbersAndColumn(
            filespec,
            self.line,
            column,
            context_before,
            context_after,
            current_line_prefix,
            stream,
        )

        #    2
        #    3    int main(int argc, char const *argv[]) {
        # => 4        printf("Hello world.\n"); // Set break point at this line.
        #    5        return 0;
        #    6    }
        self.expect(
            stream.GetData(),
            "Source code displayed correctly:\n" + stream.GetData(),
            exe=False,
            ordered=False,
            patterns=["=>", "%d.*Hello world" % self.line, needle_regex],
        )

        # Boundary condition testings for SBStream().  LLDB should not crash!
        stream.Print(None)
        stream.RedirectToFile(None, True)

    @add_test_categories(["pyapi"])
    def test_display_source_python_dumb_terminal(self):
        """Test display of source using the SBSourceManager API, using a
        dumb terminal and thus no color support (the default)."""
        use_color = False
        self.do_display_source_python_api(use_color, r"\s+\^")

    @add_test_categories(["pyapi"])
    def test_display_source_python_ansi_terminal(self):
        """Test display of source using the SBSourceManager API, using a
        dumb terminal and thus no color support (the default)."""
        use_color = True
        underline_regex = ansi_underline_surround_regex(r"printf")
        self.do_display_source_python_api(use_color, underline_regex)

    @add_test_categories(["pyapi"])
    def test_display_source_python_ansi_terminal_syntax_highlighting(self):
        """Test display of source using the SBSourceManager API and check for
        the syntax highlighted output"""
        use_color = True
        syntax_highlighting = True

        # Just pick 'int' as something that should be colored.
        color_regex = ansi_color_surround_regex("int")
        self.do_display_source_python_api(use_color, color_regex, syntax_highlighting)

        # Same for 'char'.
        color_regex = ansi_color_surround_regex("char")
        self.do_display_source_python_api(use_color, color_regex, syntax_highlighting)

        # Test that we didn't color unrelated identifiers.
        self.do_display_source_python_api(use_color, r" main\(", syntax_highlighting)
        self.do_display_source_python_api(use_color, r"\);", syntax_highlighting)

    def test_move_and_then_display_source(self):
        """Test that target.source-map settings work by moving main.c to hidden/main.c."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Move main.c to hidden/main.c.
        hidden = self.getBuildArtifact("hidden")
        lldbutil.mkdir_p(hidden)
        main_c_hidden = os.path.join(hidden, "main-copy.c")
        os.rename(self.file, main_c_hidden)

        # Set source remapping with invalid replace path and verify we get an
        # error
        self.expect(
            "settings set target.source-map /a/b/c/d/e /q/r/s/t/u",
            error=True,
            substrs=['''error: the replacement path doesn't exist: "/q/r/s/t/u"'''],
        )

        # 'make -C' has resolved current directory to its realpath form.
        builddir_real = os.path.realpath(self.getBuildDir())
        hidden_real = os.path.realpath(hidden)
        # Set target.source-map settings.
        self.runCmd(
            "settings set target.source-map %s %s" % (builddir_real, hidden_real)
        )
        # And verify that the settings work.
        self.expect(
            "settings show target.source-map", substrs=[builddir_real, hidden_real]
        )

        # Display main() and verify that the source mapping has been kicked in.
        self.expect(
            "source list -n main", SOURCE_DISPLAYED_CORRECTLY, substrs=["Hello world"]
        )

    @skipIf(oslist=["windows"], bugnumber="llvm.org/pr44431")
    def test_modify_source_file_while_debugging(self):
        """Modify a source file while debugging the executable."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main-copy.c", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=[
                "stopped",
                "main-copy.c:%d" % self.line,
                "stop reason = breakpoint",
            ],
        )

        # Display some source code.
        self.expect(
            "source list -f main-copy.c -l %d" % self.line,
            SOURCE_DISPLAYED_CORRECTLY,
            substrs=["Hello world"],
        )

        # Do the same thing with a file & line spec:
        self.expect(
            "source list -y main-copy.c:%d" % self.line,
            SOURCE_DISPLAYED_CORRECTLY,
            substrs=["Hello world"],
        )

        # The '-b' option shows the line table locations from the debug information
        # that indicates valid places to set source level breakpoints.

        # The file to display is implicit in this case.
        self.runCmd("source list -l %d -c 3 -b" % self.line)
        output = self.res.GetOutput().splitlines()[0]

        # If the breakpoint set command succeeded, we should expect a positive number
        # of breakpoints for the current line, i.e., self.line.
        import re

        m = re.search(r"^\[(\d+)\].*// Set break point at this line.", output)
        if not m:
            self.fail("Fail to display source level breakpoints")
        self.assertGreater(int(m.group(1)), 0)

        # Modify content
        self.modify_content()

        # Display the source code again. We should not see the updated line.
        self.expect(
            "source list -f main-copy.c -l %d" % self.line,
            SOURCE_DISPLAYED_CORRECTLY,
            substrs=["Hello world"],
        )

        # clear the source cache.
        self.runCmd("source cache clear")

        # Display the source code again. Now we should see the updated line.
        self.expect(
            "source list -f main-copy.c -l %d" % self.line,
            SOURCE_DISPLAYED_CORRECTLY,
            substrs=["Hello lldb"],
        )

    def test_set_breakpoint_with_absolute_path(self):
        self.build()
        hidden = self.getBuildArtifact("hidden")
        lldbutil.mkdir_p(hidden)
        # 'make -C' has resolved current directory to its realpath form.
        builddir_real = os.path.realpath(self.getBuildDir())
        hidden_real = os.path.realpath(hidden)
        self.runCmd(
            "settings set target.source-map %s %s" % (builddir_real, hidden_real)
        )

        exe = self.getBuildArtifact("a.out")
        main = os.path.join(builddir_real, "hidden", "main-copy.c")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, main, self.line, num_expected_locations=1, loc_exact=False
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=[
                "stopped",
                "main-copy.c:%d" % self.line,
                "stop reason = breakpoint",
            ],
        )

    def test_artificial_source_location(self):
        src_file = "artificial_location.cpp"
        d = {"C_SOURCES": "", "CXX_SOURCES": src_file}
        self.build(dictionary=d)

        target = lldbutil.run_to_breakpoint_make_target(self)

        # Find the instruction with line=0 and put a breakpoint there.
        sc_list = target.FindFunctions("A::foo")
        self.assertEqual(len(sc_list), 1)
        insns = sc_list[0].function.GetInstructions(target)
        insn0 = next(filter(lambda insn: insn.addr.line_entry.line == 0, insns))
        bkpt = target.BreakpointCreateBySBAddress(insn0.addr)
        self.assertGreater(bkpt.GetNumLocations(), 0)

        lldbutil.run_to_breakpoint_do_run(self, target, bkpt)

        self.expect(
            "process status",
            substrs=[
                "stop reason = breakpoint",
                f"{src_file}:0",
                "static int foo();",
                "note: This address is not associated with a specific line "
                "of code. This may be due to compiler optimizations.",
            ],
        )

    def test_source_cache_dump_and_clear(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)
        lldbutil.run_break_set_by_file_and_line(
            self, self.file, self.line, num_expected_locations=1, loc_exact=True
        )
        self.runCmd("run", RUN_SUCCEEDED)

        # Make sure the main source file is in the source cache.
        self.expect(
            "source cache dump",
            substrs=["Modification time", "Lines", "Path", " 7", self.file],
        )

        # Clear the cache.
        self.expect("source cache clear")

        # Make sure the main source file is no longer in the source cache.
        self.expect("source cache dump", matching=False, substrs=[self.file])

    def test_source_cache_interactions(self):
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Create a first target.
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)
        lldbutil.run_break_set_by_symbol(self, "main", num_expected_locations=1)
        self.expect("run", RUN_SUCCEEDED, substrs=["Hello world"])

        # Create a second target.
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)
        lldbutil.run_break_set_by_symbol(self, "main", num_expected_locations=1)
        self.expect("run", RUN_SUCCEEDED, substrs=["Hello world"])

        # Modify the source file content.
        self.modify_content()

        # Clear the source cache. This will wipe the debugger and the process
        # cache for the second process.
        self.runCmd("source cache clear")

        # Make sure we're seeing the new content from the clean process cache.
        self.expect(
            "next",
            SOURCE_DISPLAYED_CORRECTLY,
            substrs=["Hello lldb"],
        )

        # Switch back to the first target.
        self.runCmd("target select 0")

        # Make sure we're seeing the old content from the first target's
        # process cache.
        self.expect(
            "next",
            SOURCE_DISPLAYED_CORRECTLY,
            substrs=["Hello world"],
        )
